@@ -26,9 +26,20 @@
  (special lop rop $inchar)
  (*expr wxxml-lbp wxxml-rbp))
 
-($put '$wxmaxima `((mlist simp) 0 8 2) '$version)
+;;; Muffle compiler-notes globally
+#+sbcl (declaim (sb-ext:muffle-conditions sb-ext:compiler-note))
+(defmacro no-warning (form)
+  #+sbcl `(handler-bind
+	     ((style-warning #'muffle-warning)
+	      #+sbcl (sb-ext:compiler-note #'muffle-warning))
+	   ,form)
+  #+clisp `(let ((custom:*suppress-check-redefinition* t)) ,form)
+  #-(or sbcl clisp) `(progn ,form))
+
+($put '$wxmaxima `((mlist simp) 0 8 4) '$version)
 
 (setf (get '$inchar 'assign) 'neverset)
+(setf (get '$outchar 'assign) 'neverset)
 
 (defvar *var-tag* '("<v>" "</v>"))
 
@@ -204,9 +215,9 @@
 		     (if $stardisp
 			 "<t>*</t>"
 			 "<h>*</h>"))
-	       ((eq (get op 'dimension) 'dimension-nary)
-		(wxxml-dissym-to-string (get op 'dissym)))
-	       ((wxxmlsym op))))
+		     ;((wxxmlsym op))
+		     ((eq (get op 'dimension) 'dimension-nary)
+		      (wxxml-dissym-to-string (get op 'dissym)))))
          (y (cdr x))
          (ext-lop lop)
          (ext-rop rop))
@@ -218,7 +229,7 @@
                   (rop op (if (null (cdr y)) ext-rop op)))
                  ((null (cdr y))
                   (setq nl (nconc nl (wxxml (car y) l r lop rop))) nl)
-	       (setq nl (nconc nl (wxxml (car y)  l (list sym)   lop rop))
+	       (setq nl (nconc nl (wxxml (car y) l (list sym) lop rop))
 		     y (cdr y)
 		     l nil))))))
 
@@ -481,7 +492,7 @@
    ;; The exponent part
     (setq r (if (mmminusp xexp)
                 ;; the change in base-line makes parens unnecessary
-                (wxxml (cadr xexp) '("</r><r>-")
+                (wxxml (cadr xexp) '("<r><v>-</v>")
                        (cons "</r></ie>" r) 'mparen 'mparen)
 		(if (and (integerp xexp) (< xexp 10))
 		    (wxxml xexp nil
@@ -499,7 +510,7 @@
                    nil lop (caar x))
           r (if (mmminusp (setq x (nformat (caddr x))))
                 ;; the change in base-line makes parens unnecessary
-                (wxxml (cadr x) '("</r><r>-")
+                (wxxml (cadr x) '("</r><r><v>-</v>")
                        (cons "</r></e>" r) 'mminus 'mminus)
 		(if (and (integerp x) (< x 10))
 		    (wxxml x (list "</r>")
@@ -586,7 +597,7 @@
 ;; easily extended to union, intersect, otherops
 
 (defun wxxml-lsum(x l r)
-  (let ((op "<sm><r>")
+  (let ((op "<sm type=\"lsum\"><r>")
 	;; gotta be one of those above
 	(s1 (wxxml (cadr x) nil nil 'mparen rop));; summand
 	(index ;; "index = lowerlimit"
@@ -650,7 +661,7 @@
     (case (fifth x)
       ($plus
        (append l `("<lm><fnm>lim</fnm><r>"
-		   ,@subfun "<t>+</t></r><r>"
+		   ,@subfun "<v>+</v></r><r>"
 		   ,@s1 "</r></lm>") r))
       ($minus
        (append l `("<lm><fnm>lim</fnm><r>"
@@ -695,7 +706,7 @@
 
 (defun wxxml-mplus (x l r)
   (cond ((member 'trunc (car x) :test #'eq)
-	 (setq r (cons "<t>+</t><t>...</t>" r))))
+	 (setq r (cons "<v>+</v><t>...</t>" r))))
   (cond ((null (cddr x))
          (if (null (cdr x))
              (wxxml-function x l r)
@@ -705,19 +716,19 @@
            (do ((nl l)  (dissym))
                ((null (cdr x))
                 (if (mmminusp (car x)) (setq l (cadar x) dissym
-                                             (list "<t>-</t>"))
-		    (setq l (car x) dissym (list "<t>+</t>")))
+                                             (list "<v>-</v>"))
+		    (setq l (car x) dissym (list "<v>+</v>")))
                 (setq r (wxxml l dissym r 'mplus rop))
                 (append nl r))
 	     (if (mmminusp (car x)) (setq l (cadar x) dissym
-					  (list "<t>-</t>"))
-                 (setq l (car x) dissym (list "<t>+</t>")))
+					  (list "<v>-</v>"))
+                 (setq l (car x) dissym (list "<v>+</v>")))
 	     (setq nl (append nl (wxxml l dissym nil 'mplus 'mplus))
 		   x (cdr x))))))
 
 (defprop mminus wxxml-prefix wxxml)
-(defprop mminus ("-") wxxmlsym)
-(defprop mminus "<t>-</t>" wxxmlword)
+(defprop mminus ("<v>-</v>") wxxmlsym)
+(defprop mminus "<v>-</v>" wxxmlword)
 (defprop mminus 101. wxxml-rbp)
 (defprop mminus 101. wxxml-lbp)
 
@@ -734,8 +745,8 @@
 (defprop min 80. wxxml-rbp)
 
 (defprop mequal wxxml-infix wxxml)
-(defprop mequal ("<t>=</t>") wxxmlsym)
-(defprop mequal "<t>=</t>" wxxmlword)
+(defprop mequal ("<v>=</v>") wxxmlsym)
+(defprop mequal "<v>=</v>" wxxmlword)
 (defprop mequal 80. wxxml-lbp)
 (defprop mequal 80. wxxml-rbp)
 
@@ -1008,16 +1019,10 @@
 		  (setq p (make-list n :initial-element "'")))
 		 (t
 		  (setq p (list "(" n ")"))))
-	   (cond ((eq rop 'mexpt)
-		  (append l (list "<p><e><r>")
-                          (wxxml (cadr x) nil nil lop rop)
-			  (list "</r><r>") p
-                          (list "</r></e>") (list "</p>") r))
-		 (t
-		  (append (append l '("<e><r>"))
-                          (wxxml (cadr x) nil nil lop rop)
-			  (list "</r><r>") p
-                          (list "</r></e>")  r)))))
+	   (append (append l '("<r>"))
+		   (let ((*var-tag* (list "<fnm>" "</fnm>"))) (wxxml (cadr x) nil nil lop rop))
+		   p
+		   (list "</r>")  r)))
 
 	((and $pdiff_uses_named_subscripts_for_derivatives
 	      (< (apply #'+ (cddr x)) $pdiff_prime_limit))
@@ -1073,27 +1078,38 @@
 	    ($first $wxplot_size)
 	    ($second $wxplot_size))))
 
-(defun $with_slider (&rest args)
-  (apply #'$wxanimate args))
+(defun $int_range (lo &optional hi)
+  (unless (integerp lo)
+    ($error "int_range: first argument is not an integer."))
+  (unless (or (null hi) (integerp hi))
+    ($error "int_range: second argument is not an integer."))
+  (when (null hi)
+    (setq hi lo)
+    (setq lo 1))
+  (cons '(mlist simp) (loop for i from lo to hi collect i)))
 
-(defun $wxanimate (a a-range expr &rest args)
-  (let (images)
+(defun wxanimate (scene)
+  (let* ((scene (cdr scene))
+	 (a (car scene))
+	 (a-range (meval (cadr scene)))
+	 (expr (caddr scene))
+	 (args (cdddr scene))
+	 (images ()))
+    (when (integerp a-range)
+      (setq a-range (mfuncall '$makelist 'i 'i 1 a-range)))
     (dolist (aval (reverse (cdr a-range)))
       (let ((preamble ($wxplot_preamble))
 	    (system-preamble (get-plot-option-string '$gnuplot_preamble 2))
 	    (filename (wxplot-filename))
 	    (expr (maxima-substitute aval a expr)))
-	(setq preamble
-	      (format nil "set title '~a=~a'~%~a"
-		      (stripdollar (maybe-invert-string-case (symbol-name a)))
-		      (mfuncall '$string aval)
-		      preamble))
+	(when (string= system-preamble "false")
+	  (setq system-preamble ""))
 	(setq preamble (format nil "~a; ~a" preamble system-preamble))
 	(dolist (arg args)
 	  (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
 	      (setq preamble (format nil "~a; ~a"
 				     preamble (caddr arg)))))
-	(apply #'$plot2d `(,expr ,@args
+	(apply #'$plot2d `(,(meval expr) ,@(mapcar #'meval args)
 			   ((mlist simp) $plot_format $gnuplot)
 			   ((mlist simp) $gnuplot_preamble ,preamble)
 			   ((mlist simp) $gnuplot_term $png)
@@ -1102,28 +1118,32 @@
     ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))
   "")
 
-(defvar *windows-OS* (string= *autoconf-win32* "true"))
+(defmspec $with_slider (scene)
+  (wxanimate scene))
 
-(defun $with_slider_draw (&rest args)
-  (apply #'$wxanimate_draw args))
+(defmspec $wxanimate (scene)
+  (wxanimate scene))
+
+(defvar *windows-OS* (string= *autoconf-win32* "true"))
 
 (defun draw-transform (arg trans)
   (declare (ignore trans))
   arg)
 
-(defun $wxanimate_draw (a a-range &rest args)
-  (let (images)
+(defun wxanimate-draw (scene)
+  (let* ((scene (cdr scene))
+	 (a (meval (car scene)))
+	 (a-range (meval (cadr scene)))
+	 (args (cddr scene))
+	 (images ()))
+    (when (integerp a-range)
+      (setq a-range (mfuncall '$makelist 'i 'i 1 a-range)))
     (dolist (aval (reverse (cdr a-range)))
       (let* ((filename (wxplot-filename nil))
 	     (*windows-OS* t)
-	     (args (cons `((mequal simp) $title
-			   ,(format nil "~a=~a"
-				   (stripdollar (maybe-invert-string-case (symbol-name a)))
-				   (mfuncall '$string aval)))
-			 args))
 	     (args (cons '($gr2d)
 			 (draw-transform
-			  (mapcar #'(lambda (arg) (maxima-substitute aval a arg))
+			  (mapcar #'(lambda (arg) (meval (maxima-substitute aval a arg)))
 				  args)
 			  '$draw2d_transform))))
 	(setq images (cons (format nil "~a.png" filename) images))
@@ -1138,22 +1158,26 @@
     ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))
   "")
 
-(defun $with_slider_draw3d (&rest args)
-  (apply #'$wxanimate_draw3d args))
+(defmspec $wxanimate_draw (scene)
+  (wxanimate-draw scene))
 
-(defun $wxanimate_draw3d (a a-range &rest args)
-  (let (images)
+(defmspec $with_slider_draw (scene)
+  (wxanimate-draw scene))
+
+(defun wxanimate-draw3d (scene)
+  (let* ((scene (cdr scene))
+	 (a (meval (car scene)))
+	 (a-range (meval (cadr scene)))
+	 (args (cddr scene))
+	 (images ()))
+    (when (integerp a-range)
+      (setq a-range (mfuncall '$makelist 'i 'i 1 a-range)))
     (dolist (aval (reverse (cdr a-range)))
       (let* ((filename (wxplot-filename nil))
 	     (*windows-OS* t)
-	     (args (cons `((mequal simp) $title
-			   ,(format nil "~a=~a"
-				   (stripdollar (maybe-invert-string-case (symbol-name a)))
-				   (mfuncall '$string aval)))
-			 args))
 	     (args (cons '($gr3d)
 			 (draw-transform
-			  (mapcar #'(lambda (arg) (maxima-substitute aval a arg))
+			  (mapcar #'(lambda (arg) (meval (maxima-substitute aval a arg)))
 				  args)
 			  '$draw3d_transform))))
 	(setq images (cons (format nil "~a.png" filename) images))
@@ -1168,10 +1192,18 @@
     ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))
   "")
 
+(defmspec $with_slider_draw3d (scene)
+  (wxanimate-draw3d scene))
+
+(defmspec $wxanimate_draw3d (scene)
+  (wxanimate-draw3d scene))
+
 (defun $wxplot2d (&rest args)
   (let ((preamble ($wxplot_preamble))
 	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
 	(filename (wxplot-filename)))
+    (when (string= system-preamble "false")
+      (setq system-preamble ""))
     (setq preamble (format nil "~a; ~a" preamble system-preamble))
     (dolist (arg args)
       (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
@@ -1189,6 +1221,8 @@
   (let ((preamble ($wxplot_preamble))
 	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
 	(filename (wxplot-filename)))
+    (when (string= system-preamble "false")
+      (setq system-preamble ""))
     (setq preamble (format nil "~a; ~a" preamble system-preamble))
     (dolist (arg args)
       (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
@@ -1213,6 +1247,8 @@
   (apply #'$wxdraw
 	 (list (cons '($gr3d) (draw-transform args '$draw3d_transform)))))
 
+(defvar $display_graphics t)
+
 (defun $wxdraw (&rest args)
   (let* ((filename (wxplot-filename nil))
 	 (*windows-OS* t)
@@ -1225,13 +1261,22 @@
 			((mequal simp) $pic_height ,($second $wxplot_size))
 			((mequal simp) $file_name ,filename))
 		      args)))
-    ($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"))
+    (if $display_graphics
+	($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"))
+	(setf res `((wxxmltag simp) ,(format nil "~a.png" filename) "img")))
     res))
 
+(defmspec $wxdraw_list (args)
+  (let (($display_graphics nil))
+    ($ldisp (cons '(mlist simp) (mapcar #'meval (cdr args)))))
+  '$done)
+
 (defun $wximplicit_plot (&rest args)
   (let ((preamble ($wxplot_preamble))
 	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
 	(filename (wxplot-filename)))
+    (when (string= system-preamble "false")
+      (setq system-preamble ""))
     (setq preamble (format nil "~a; ~a" preamble system-preamble))
     (dolist (arg args)
       (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
@@ -1250,6 +1295,8 @@
   (let ((preamble ($wxplot_preamble))
 	(system-preamble (get-plot-option-string '$gnuplot_preamble 2))
 	(filename (wxplot-filename)))
+    (when (string= system-preamble "false")
+      (setq system-preamble ""))
     (setq preamble (format nil "~a; ~a" preamble system-preamble))
     (dolist (arg args)
       (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
@@ -1307,3 +1354,105 @@
 	 (putprop e  s 'wxxmlsym)
          (putprop e lbp 'wxxml-lbp))
         (t (merror "Improper arguments to `wxxmlput'."))))
+
+;;;;;;;;;;;;;
+;; Auto-loaded functions
+;;;;
+
+(setf (get '$lbfgs 'autoload) "lbfgs")
+(setf (get '$lcm 'autoload) "functs")
+
+;;;;;;;;;;;;;
+;; Statistics functions
+;;;;
+
+(defvar $draw_compound t)
+
+(defmacro create-statistics-wrapper (fun wxfun)
+  `(defun ,wxfun (&rest args)
+     (let (($draw_compound nil) res)
+       (declare (special $draw_compound))
+       (setq res ($apply ',fun (cons '(mlist simp) args)))
+       ($apply '$wxdraw2d res))))
+
+(create-statistics-wrapper $histogram $wxhistogram)
+(create-statistics-wrapper $scatterplot $wxscatterplot)
+(create-statistics-wrapper $barsplot $wxbarsplot)
+(create-statistics-wrapper $piechart $wxpiechart)
+(create-statistics-wrapper $boxplot $wxboxplot)
+
+(dolist (fun '($histogram
+	       $scatterplot
+	       $barsplot
+	       $piechart
+	       $boxplot))
+  (setf (get fun 'autoload) "descriptive"))
+
+(dolist (fun '($mean
+	       $median
+	       $var
+	       $std
+	       $test_mean
+	       $test_means_difference
+	       $test_normality
+	       $simple_linear_regression
+	       $subsample))
+  (setf (get fun 'autoload) "stats"))
+
+(setf (get '$lsquares_estimates 'autoload) "lsquares")
+
+(setf (get '$to_poly_solve 'autoload) "to_poly_solver")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;
+;; Redefine load so that it prints the list of functions
+;; used for autocompletion.
+;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defun symbol-to-string (s)
+  (maybe-invert-string-case (symbol-name (stripdollar s))))
+
+(defun $print_function (fun)
+  (let ((fun-name (symbol-to-string (caar fun)))
+	(args (mapcar (lambda (u)
+			(cond ((atom u) (symbol-to-string u))
+			      ((eq (caar u) 'mlist)
+			       ($concat "[" (symbol-to-string (cadr u)) "]"))
+			      (t (symbol-to-string (cadr u)))))
+		      (cdr fun))))
+    (format nil "FUNCTION: ~a$TEMPLATE: ~a(~{<~a>~^, ~})" fun-name fun-name args)))
+
+(defun $add_function_template (&rest functs)
+  (format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>" (mapcar #'$print_function functs))
+  (cons '(mlist simp) functs))
+
+;;;
+;;; Rewrite of the function load (maxima/src/mload.lisp)
+;;; (displays functions after loading a maxima package
+;;;
+(no-warning
+ (defun $load (filename)
+   (let ((searched-for
+	  ($file_search1 filename
+			 '((mlist) $file_search_maxima $file_search_lisp  )))
+	 type)
+     (setq type ($file_type searched-for))
+     (case type
+       (($maxima)
+	($batchload searched-for)
+	(format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>"
+		(mapcar #'$print_function (cdr ($append $functions $macros)))))
+       (($lisp $object)
+	;; do something about handling errors
+	;; during loading. Foobar fail act errors.
+	(load-and-tell searched-for))
+       (t
+	(merror "Maxima bug: Unknown file type ~M" type)))
+     searched-for)))
+
+;; Load the initial functions (from mac-init.mac)
+(format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>"
+	(mapcar #'$print_function (cdr ($append $functions $macros))))
+
+(when ($file_search "wxmaxima-init")
+  ($load "wxmaxima-init"))
