@@ -1,5 +1,6 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///            (C) 2008-2009 Ziga Lenarcic <zigalenarcic@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -37,6 +38,8 @@ const int langs[] =
   {
     wxLANGUAGE_DEFAULT,
     wxLANGUAGE_CHINESE_TRADITIONAL,
+    wxLANGUAGE_CZECH,
+    wxLANGUAGE_DANISH,
     wxLANGUAGE_ENGLISH,
     wxLANGUAGE_FRENCH,
     wxLANGUAGE_GERMAN,
@@ -49,15 +52,14 @@ const int langs[] =
     wxLANGUAGE_UKRAINIAN
   };
 
-#define LANGUAGE_NUMBER 12
+#define LANGUAGE_NUMBER 14
 
 Config::Config(wxWindow* parent, int id, const wxString& title,
                const wxPoint& pos, const wxSize& size, long style):
     wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE)
 {
-  int defaultPort = 4010, adj = 0;
+  int defaultPort = 4010;
   wxConfig::Get()->Read(wxT("defaultPort"), &defaultPort);
-  wxConfig::Get()->Read(wxT("Style/GreekFont/adj"), &adj);
 
   notebook_1 = new wxNotebook(this, -1, wxDefaultPosition, wxDefaultSize, 0);
   notebook_1_pane_2 = new wxPanel(notebook_1, -1);
@@ -76,6 +78,8 @@ Config::Config(wxWindow* parent, int id, const wxString& title,
     {
       _("(Use default language)"),
       _("Chinese traditional"),
+      _("Czech"),
+      _("Danish"),
       _("English"),
       _("French"),
       _("German"),
@@ -88,33 +92,21 @@ Config::Config(wxWindow* parent, int id, const wxString& title,
       _("Ukrainian")
     };
   m_language = new wxComboBox(notebook_1_pane_1, language_id, wxEmptyString, wxDefaultPosition, wxSize(230, -1), LANGUAGE_NUMBER, m_language_choices, wxCB_DROPDOWN | wxCB_READONLY);
-  label_9 = new wxStaticText(notebook_1_pane_1, -1, _("Button panel:"));
-  const wxString m_panelSize_choices[] =
-    {
-      _("Off"), _("Basic"), _("Full")
-    };
   label_12 = new wxStaticText(notebook_1_pane_1, -1, _("Default port:"));
   m_defaultPort = new wxSpinCtrl(notebook_1_pane_1, -1, wxEmptyString, wxDefaultPosition, wxSize(70, -1), wxSP_ARROW_KEYS, 50, 5000, defaultPort);
   m_defaultPort->SetValue(defaultPort);
-  m_panelSize = new wxComboBox(notebook_1_pane_1, panel_size, wxEmptyString, wxDefaultPosition, wxSize(230, -1), 3, m_panelSize_choices, wxCB_DROPDOWN | wxCB_READONLY);
   m_saveSize = new wxCheckBox(notebook_1_pane_1, -1, _("Save wxMaxima window size/position"));
+  m_savePanes = new wxCheckBox(notebook_1_pane_1, -1, _("Save panes layout"));
   m_matchParens = new wxCheckBox(notebook_1_pane_1, -1, _("Match parenthesis in text controls"));
   m_fixedFontInTC = new wxCheckBox(notebook_1_pane_1, -1, _("Fixed font in text controls"));
   m_showLong = new wxCheckBox(notebook_1_pane_1, -1, _("Show long expressions"));
-  m_showHeader = new wxCheckBox(notebook_1_pane_1, -1, _("Show Maxima header"));
-  m_unixCopy = new wxCheckBox(notebook_1_pane_1, -1, _("Copy to clipboard on select"));
   m_changeAsterisk = new wxCheckBox(notebook_1_pane_1, -1, _("Use centered dot character for multiplication"));
   m_enterEvaluates = new wxCheckBox(notebook_1_pane_1, -1, _("Enter evaluates cells"));
   label_8 = new wxStaticText(notebook_1_pane_2, -1, _("Default font:"));
   m_getFont = new wxButton(notebook_1_pane_2, font_family, _("Choose font"), wxDefaultPosition, wxSize(250, -1));
-  m_greekFontOk = new wxCheckBox(notebook_1_pane_2, checkbox_greek, _("Use greek font:"));
-  m_getGreekFont = new wxButton(notebook_1_pane_2, button_greek, _("Choose font"), wxDefaultPosition, wxSize(250, -1));
-  label_10 = new wxStaticText(notebook_1_pane_2, -1, _("Adjustment:"));
-  m_greekFontAdj = new wxSpinCtrl(notebook_1_pane_2, -1, wxEmptyString, wxDefaultPosition, wxSize(70, -1), wxSP_ARROW_KEYS, -4, 4, adj);
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-  m_unicodeGlyphs = new wxStaticText(notebook_1_pane_2, -1, _("Unicode glyphs:"));
-  m_getUnicodeFont = new wxButton(notebook_1_pane_2, unicode_glyphs, _("Choose font"), wxDefaultPosition, wxSize(250, -1));
-#endif
+  m_mathFont = new wxStaticText(notebook_1_pane_2, -1, _("Math font:"));
+  m_getMathFont = new wxButton(notebook_1_pane_2, button_mathFont, _("Choose font"), wxDefaultPosition, wxSize(250, -1));
+  m_useJSMath = new wxCheckBox(notebook_1_pane_2, -1, _("Use jsMath fonts"));
   const wxString m_styleFor_choices[] =
     {
       _("Default"),
@@ -130,6 +122,7 @@ Config::Config(wxWindow* parent, int id, const wxString& title,
       _("Output labels"),
       _("Highlight (dpart)"),
       _("Text cell"),
+      _("Subsection cell"),
       _("Section cell"),
       _("Title cell"),
       _("Text cell background"),
@@ -140,7 +133,7 @@ Config::Config(wxWindow* parent, int id, const wxString& title,
       _("Selection"),
       _("Outdated cells")
     };
-  m_styleFor = new wxListBox(notebook_1_pane_2, listbox_styleFor, wxDefaultPosition, wxSize(200, -1), 22, m_styleFor_choices, wxLB_SINGLE);
+  m_styleFor = new wxListBox(notebook_1_pane_2, listbox_styleFor, wxDefaultPosition, wxSize(200, -1), 23, m_styleFor_choices, wxLB_SINGLE);
   m_getStyleFont = new wxButton(notebook_1_pane_2, style_font_family, _("Choose font"), wxDefaultPosition, wxSize(150, -1));
 #ifndef __WXMSW__
   m_styleColor = new ColorPanel(this, notebook_1_pane_2, color_id, wxDefaultPosition, wxSize(150, 30), wxSUNKEN_BORDER | wxFULL_REPAINT_ON_RESIZE);
@@ -174,27 +167,20 @@ void Config::set_properties()
   m_additionalParameters->SetToolTip(_("Additional parameters for Maxima"
                                        " (e.g. -l clisp)."));
   m_saveSize->SetToolTip(_("Save wxMaxima window size/position between sessions."));
+  m_savePanes->SetToolTip(_("Save panes layout between sessions."));
   m_matchParens->SetToolTip(_("Write matching parenthesis in text controls."));
   m_showLong->SetToolTip(_("Show long expressions in wxMaxima document."));
   m_language->SetToolTip(_("Language used for wxMaxima GUI."));
-  m_showHeader->SetToolTip(_("Show initial header with Maxima system information."));
-  m_fixedFontInTC->SetToolTip(_("Set fixed font in text controls."));
   m_fixedFontInTC->SetToolTip(_("Set fixed font in text controls."));
   m_getFont->SetToolTip(_("Font used for display in document."));
-  m_greekFontOk->SetToolTip(_("Use greek font to display greek characters."));
-  m_getGreekFont->SetToolTip(_("Font used for displaying greek characters in document."));
-  m_greekFontAdj->SetToolTip(_("Adjustment for the size of greek font."));
-  m_unixCopy->SetToolTip(_("Copy selection to clipboard when selection is made in document."));
+  m_getMathFont->SetToolTip(_("Font used for displaying math characters in document."));
   m_changeAsterisk->SetToolTip(_("Use centered dot character for multiplication"));
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-  m_getUnicodeFont->SetToolTip(_("Font used for displaying unicode glyphs in document."));
-#endif
   m_defaultPort->SetToolTip(_("The default port used for communication between Maxima and wxMaxima."));
 
   wxConfig *config = (wxConfig *)wxConfig::Get();
   wxString mp, mc, ib, mf;
-  bool match = true, showLongExpr = false, unixCopy = false, activateSelection = true;
-  bool showHeader = false, fixedFontTC = true, readFile = false, changeAsterisk = false;
+  bool match = true, showLongExpr = false, savePanes = false;
+  bool fixedFontTC = true, changeAsterisk = false, usejsmath = true;
   bool enterEvaluates = false;
   int rs = 0;
   int lang = wxLANGUAGE_UNKNOWN;
@@ -202,16 +188,16 @@ void Config::set_properties()
 
   config->Read(wxT("maxima"), &mp);
   config->Read(wxT("parameters"), &mc);
+  config->Read(wxT("AUI/savePanes"), &savePanes);
   config->Read(wxT("pos-restore"), &rs);
   config->Read(wxT("matchParens"), &match);
   config->Read(wxT("showLong"), &showLongExpr);
   config->Read(wxT("language"), &lang);
-  config->Read(wxT("showHeader"), &showHeader);
-  config->Read(wxT("unixCopy"), &unixCopy);
   config->Read(wxT("changeAsterisk"), &changeAsterisk);
   config->Read(wxT("fixedFontTC"), &fixedFontTC);
   config->Read(wxT("panelSize"), &panelSize);
   config->Read(wxT("enterEvaluates"), &enterEvaluates);
+  config->Read(wxT("usejsmath"), &usejsmath);
 
   int i = 0;
   for (i = 0; i < LANGUAGE_NUMBER; i++)
@@ -222,8 +208,6 @@ void Config::set_properties()
   else
     m_language->SetSelection(0);
 
-  m_panelSize->SetSelection(panelSize);
-
 #if defined __WXMSW__
   wxString cwd = wxGetCwd();
   cwd.Replace(wxT("wxMaxima"), wxT("\\bin\\maxima.bat"));
@@ -257,13 +241,13 @@ void Config::set_properties()
     m_saveSize->SetValue(true);
   else
     m_saveSize->SetValue(false);
+  m_savePanes->SetValue(savePanes);
   m_matchParens->SetValue(match);
   m_showLong->SetValue(showLongExpr);
-  m_showHeader->SetValue(showHeader);
-  m_unixCopy->SetValue(unixCopy);
   m_changeAsterisk->SetValue(changeAsterisk);
   m_enterEvaluates->SetValue(enterEvaluates);
   m_fixedFontInTC->SetValue(fixedFontTC);
+  m_useJSMath->SetValue(usejsmath);
 
   m_getStyleFont->Enable(false);
 
@@ -273,6 +257,13 @@ void Config::set_properties()
   m_button2->SetDefault();
 #endif
 
+  if (!wxFontEnumerator::IsValidFacename(wxT("jsMath-cmex10")) ||
+      !wxFontEnumerator::IsValidFacename(wxT("jsMath-cmsy10")) ||
+      !wxFontEnumerator::IsValidFacename(wxT("jsMath-cmr10")) ||
+      !wxFontEnumerator::IsValidFacename(wxT("jsMath-cmmi10")) ||
+      !wxFontEnumerator::IsValidFacename(wxT("jsMath-cmti10")))
+    m_useJSMath->Enable(false);
+
   ReadStyles();
 }
 
@@ -308,17 +299,14 @@ void Config::do_layout()
   // wxMaxima options box
   grid_sizer_5->Add(label_4, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
   grid_sizer_5->Add(m_language, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_5->Add(label_9, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_5->Add(m_panelSize, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
   grid_sizer_5->Add(label_12, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
   grid_sizer_5->Add(m_defaultPort, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
   sizer_6->Add(grid_sizer_5, 1, wxEXPAND, 0);
   sizer_6->Add(m_saveSize, 0, wxALL, 3);
+  sizer_6->Add(m_savePanes, 0, wxALL, 3);
   sizer_6->Add(m_matchParens, 0, wxALL, 3);
   sizer_6->Add(m_fixedFontInTC, 0, wxALL, 3);
   sizer_6->Add(m_showLong, 0, wxALL, 3);
-  sizer_6->Add(m_showHeader, 0, wxALL, 3);
-  sizer_6->Add(m_unixCopy, 0, wxALL, 3);
   sizer_6->Add(m_changeAsterisk, 0, wxALL, 3);
   sizer_6->Add(m_enterEvaluates, 0, wxALL, 3);
   sizer_3->Add(sizer_6, 1, wxALL | wxEXPAND, 3);
@@ -333,14 +321,10 @@ void Config::do_layout()
   // Font box
   grid_sizer_1->Add(label_8, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
   grid_sizer_1->Add(m_getFont, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_1->Add(m_greekFontOk, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_1->Add(m_getGreekFont, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_1->Add(label_10, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_1->Add(m_greekFontAdj, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-  grid_sizer_1->Add(m_unicodeGlyphs, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
-  grid_sizer_1->Add(m_getUnicodeFont, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
-#endif
+  grid_sizer_1->Add(m_mathFont, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
+  grid_sizer_1->Add(m_getMathFont, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
+  grid_sizer_1->Add(10, 10);
+  grid_sizer_1->Add(m_useJSMath, 0, wxALL | wxALIGN_CENTER_VERTICAL, 3);
   sizer_9->Add(grid_sizer_1, 1, wxALL | wxEXPAND, 3);
   sizer_8->Add(sizer_9, 1, wxALL | wxEXPAND, 3);
 
@@ -400,15 +384,15 @@ void Config::OnOk(wxCommandEvent& event)
   config->Write(wxT("maxima"), m_maximaProgram->GetValue());
   config->Write(wxT("parameters"), m_additionalParameters->GetValue());
   config->Write(wxT("fontSize"), m_fontSize);
+  config->Write(wxT("mathFontsize"), m_mathFontSize);
   config->Write(wxT("matchParens"), m_matchParens->GetValue());
   config->Write(wxT("showLong"), m_showLong->GetValue());
-  config->Write(wxT("showHeader"), m_showHeader->GetValue());
   config->Write(wxT("fixedFontTC"), m_fixedFontInTC->GetValue());
-  config->Write(wxT("unixCopy"), m_unixCopy->GetValue());
   config->Write(wxT("changeAsterisk"), m_changeAsterisk->GetValue());
   config->Write(wxT("enterEvaluates"), m_enterEvaluates->GetValue());
-  config->Write(wxT("panelSize"), m_panelSize->GetSelection());
   config->Write(wxT("defaultPort"), m_defaultPort->GetValue());
+  config->Write(wxT("AUI/savePanes"), m_savePanes->GetValue());
+  config->Write(wxT("usejsmath"), m_useJSMath->GetValue());
   if (m_saveSize->GetValue())
     config->Write(wxT("pos-restore"), 1);
   else
@@ -454,23 +438,18 @@ void Config::OnMpBrowse(wxCommandEvent& event)
   }
 }
 
-void Config::OnGreekBrowse(wxCommandEvent& event)
+void Config::OnMathBrowse(wxCommandEvent& event)
 {
-  wxFont greek;
-#if defined __WXMSW__
-  greek = wxGetFontFromUser(this, wxFont(12, wxNORMAL, wxNORMAL, wxNORMAL,
-                                         false, m_greekFontName,
-                                         wxFONTENCODING_CP1253));
-#else
-  greek = wxGetFontFromUser(this, wxFont(12, wxNORMAL, wxNORMAL, wxNORMAL,
-                                         false, m_greekFontName,
-                                         wxFONTENCODING_ISO8859_7));
-#endif
+  wxFont math;
+  math = wxGetFontFromUser(this, wxFont(m_mathFontSize, wxNORMAL, wxNORMAL, wxNORMAL,
+                                        false, m_mathFontName));
 
-  if (greek.Ok())
+  if (math.Ok())
   {
-    m_greekFontName = greek.GetFaceName();
-    m_getGreekFont->SetLabel(m_greekFontName);
+    m_mathFontName = math.GetFaceName();
+    m_mathFontSize = math.GetPointSize();
+    m_getMathFont->SetLabel(m_mathFontName + wxString::Format(wxT(" (%d)"), m_mathFontSize));
+    UpdateExample();
   }
 }
 
@@ -481,7 +460,7 @@ void Config::OnChangeFontFamily(wxCommandEvent& event)
   style *tmp = GetStylePointer();
   wxString fontName;
 
-  if (tmp == &m_styleText || tmp == &m_styleTitle || tmp == &m_styleSection)
+  if (tmp == &m_styleText || tmp == &m_styleTitle || tmp == &m_styleSubsection || tmp == &m_styleSection)
   {
     if (tmp->fontSize != 0)
       fontsize = tmp->fontSize;
@@ -515,21 +494,6 @@ void Config::OnChangeFontFamily(wxCommandEvent& event)
   }
 }
 
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-void Config::OnChangeUnicodeFont(wxCommandEvent& event)
-{
-  wxFont font;
-  font = wxGetFontFromUser(this, wxFont(m_fontSize,
-                                        wxNORMAL, wxNORMAL, wxNORMAL,
-                                        false, m_unicodeFont));
-  if (font.Ok())
-  {
-    m_unicodeFont = font.GetFaceName();
-    m_getUnicodeFont->SetLabel(m_unicodeFont);
-  }
-}
-#endif
-
 void Config::ReadStyles(wxString file)
 {
   wxConfigBase* config;
@@ -540,36 +504,21 @@ void Config::ReadStyles(wxString file)
     config = new wxFileConfig(str);
   }
 
-  int adj = 0;
-  bool greekOk = false;
-
-  m_fontSize = 12;
+  m_fontSize = m_mathFontSize = 12;
   config->Read(wxT("fontsize"), &m_fontSize);
+  config->Read(wxT("mathfontsize"), &m_mathFontSize);
   config->Read(wxT("Style/fontname"), &m_styleDefault.font);
   if (m_styleDefault.font.Length())
     m_getFont->SetLabel(m_styleDefault.font + wxString::Format(wxT(" (%d)"), m_fontSize));
 
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-  m_unicodeFont = m_styleDefault.font;
-  config->Read(wxT("Style/Unicode/fontname"), &m_unicodeFont);
-  if (m_unicodeFont.Length())
-    m_getUnicodeFont->SetLabel(m_unicodeFont);
-#endif
-
   int encoding = wxFONTENCODING_DEFAULT;
   config->Read(wxT("fontEncoding"), &encoding);
   m_fontEncoding = (wxFontEncoding)encoding;
 
-  m_greekFontName = wxEmptyString;
-  config->Read(wxT("Style/GreekFont/ok"), &greekOk);
-  config->Read(wxT("Style/GreekFont/fontname"), &m_greekFontName);
-  config->Read(wxT("Style/GreekFont/adj"), &adj);
-  m_greekFontAdj->SetValue(adj);
-  m_greekFontOk->SetValue(greekOk);
-  if (m_greekFontName.Length() > 0)
-    m_getGreekFont->SetLabel(m_greekFontName);
-  m_getGreekFont->Enable(greekOk);
-  m_greekFontAdj->Enable(greekOk);
+  m_mathFontName = wxEmptyString;
+  config->Read(wxT("Style/Math/fontname"), &m_mathFontName);
+  if (m_mathFontName.Length() > 0)
+    m_getMathFont->SetLabel(m_mathFontName + wxString::Format(wxT(" (%d)"), m_mathFontSize));
 
   wxString tmp;
   // Document background color
@@ -715,6 +664,20 @@ void Config::ReadStyles(wxString file)
                &m_styleText.font);
   READ_STYLE(m_styleText, "Style/Text/")
 
+  // Subsection
+  m_styleSubsection.color = wxT("black");
+  m_styleSubsection.bold = true;
+  m_styleSubsection.italic = false;
+  m_styleSubsection.underlined = false;
+  m_styleSubsection.font = m_styleDefault.font;
+  m_styleSubsection.fontSize = 16;
+  config->Read(wxT("Style/Subsection/fontsize"),
+                 &m_styleSubsection.fontSize);
+  config->Read(wxT("Style/Subsection/fontname"),
+               &m_styleSubsection.font);
+  READ_STYLE(m_styleSubsection, "Style/Subsection/")
+
+
   // Section
   m_styleSection.color = wxT("black");
   m_styleSection.bold = true;
@@ -784,13 +747,7 @@ void Config::WriteStyles(wxString file)
   config->Write(wxT("Style/fontname"), m_styleDefault.font);
   config->Write(wxT("fontEncoding"), (int)m_fontEncoding);
 
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-  config->Write(wxT("Style/Unicode/fontname"), m_unicodeFont);
-#endif
-
-  config->Write(wxT("Style/GreekFont/ok"), m_greekFontOk->GetValue());
-  config->Write(wxT("Style/GreekFont/fontname"), m_greekFontName);
-  config->Write(wxT("Style/GreekFont/adj"), m_greekFontAdj->GetValue());
+  config->Write(wxT("Style/Math/fontname"), m_mathFontName);
 
 #define WRITE_STYLE(style, where)                   \
   config->Write(wxT(where "color"), style.color.GetAsString(wxC2S_CSS_SYNTAX));   \
@@ -833,6 +790,11 @@ void Config::WriteStyles(wxString file)
   config->Write(wxT("Style/Text/fontsize"), m_styleText.fontSize);
   WRITE_STYLE(m_styleText, "Style/Text/")
 
+  // Section
+  config->Write(wxT("Style/Subsection/fontname"), m_styleSubsection.font);
+  config->Write(wxT("Style/Subsection/fontsize"), m_styleSubsection.fontSize);
+  WRITE_STYLE(m_styleSubsection, "Style/Subsection/")
+
   // Section
   config->Write(wxT("Style/Section/fontname"), m_styleSection.font);
   config->Write(wxT("Style/Section/fontsize"), m_styleSection.fontSize);
@@ -876,13 +838,13 @@ void Config::OnChangeStyle(wxCommandEvent& event)
 
   m_styleColor->SetBackgroundColour(tmp->color);
 
-  if (st >= 12 && st <= 14)
+  if (st >= 12 && st <= 15)
     m_getStyleFont->Enable(true);
   else
     m_getStyleFont->Enable(false);
 
   // Background color only
-  if (st >= 15)
+  if (st >= 16)
   {
     m_boldCB->Enable(false);
     m_italicCB->Enable(false);
@@ -911,12 +873,6 @@ void Config::OnCheckbox(wxCommandEvent& event)
   UpdateExample();
 }
 
-void Config::OnCheckGreek(wxCommandEvent& event)
-{
-  m_getGreekFont->Enable(m_greekFontOk->GetValue());
-  m_greekFontAdj->Enable(m_greekFontOk->GetValue());
-}
-
 void Config::OnChangeWarning(wxCommandEvent &event)
 {
   wxMessageBox(_("Please restart wxMaxima for changes to take effect!"),
@@ -969,30 +925,33 @@ style* Config::GetStylePointer()
     tmp = &m_styleText;
     break;
   case 13:
-    tmp = &m_styleSection;
+    tmp = &m_styleSubsection;
     break;
   case 14:
-    tmp = &m_styleTitle;
+    tmp = &m_styleSection;
     break;
   case 15:
-    tmp = &m_styleTextBackground;
+    tmp = &m_styleTitle;
     break;
   case 16:
-    tmp = &m_styleBackground;
+    tmp = &m_styleTextBackground;
     break;
   case 17:
-    tmp = &m_styleCellBracket;
+    tmp = &m_styleBackground;
     break;
   case 18:
-    tmp = &m_styleActiveCellBracket;
+    tmp = &m_styleCellBracket;
     break;
   case 19:
-    tmp = &m_styleCursor;
+    tmp = &m_styleActiveCellBracket;
     break;
   case 20:
-    tmp = &m_styleSelection;
+    tmp = &m_styleCursor;
     break;
   case 21:
+    tmp = &m_styleSelection;
+    break;
+  case 22:
     tmp = &m_styleOutdated;
     break;
   }
@@ -1006,19 +965,23 @@ void Config::UpdateExample()
   wxColour color(tmp->color);
   wxString font(m_styleDefault.font);
 
-//  wxClientDC dc(label_11);
-
   if (tmp == &m_styleBackground)
       color = m_styleInput.color;
 
   int fontsize = m_fontSize;
-  if (tmp == &m_styleText || tmp == &m_styleSection || tmp == &m_styleTitle)
+  if (tmp == &m_styleText || tmp == &m_styleSubsection || tmp == &m_styleSection || tmp == &m_styleTitle)
   {
     fontsize = tmp->fontSize;
     font = tmp->font;
     if (fontsize == 0)
       fontsize = m_fontSize;
   }
+  else if (tmp == &m_styleVariable || tmp == &m_styleNumber || tmp == &m_styleFunction ||
+      tmp == &m_styleSpecial)
+  {
+    fontsize = m_mathFontSize;
+    font = m_mathFontName;
+  }
 
   if (tmp == &m_styleTextBackground) {
       label_11->SetFontSize(m_styleText.fontSize);
@@ -1029,7 +992,7 @@ void Config::UpdateExample()
   }
 
   if (tmp == &m_styleTextBackground ||
-      tmp == &m_styleText || tmp == &m_styleSection || tmp == &m_styleTitle)
+      tmp == &m_styleText)
     label_11->SetBackgroundColour(m_styleTextBackground.color);
   else
     label_11->SetBackgroundColour(m_styleBackground.color);
@@ -1071,21 +1034,16 @@ void Config::OnColorButton(wxCommandEvent &event)
 BEGIN_EVENT_TABLE(Config, wxDialog)
   EVT_BUTTON(wxID_OK, Config::OnOk)
   EVT_BUTTON(wxID_OPEN, Config::OnMpBrowse)
-  EVT_BUTTON(button_greek, Config::OnGreekBrowse)
+  EVT_BUTTON(button_mathFont, Config::OnMathBrowse)
   EVT_BUTTON(font_family, Config::OnChangeFontFamily)
-#if (wxUSE_UNICODE && WXM_UNICODE_GLYPHS)
-  EVT_BUTTON(unicode_glyphs, Config::OnChangeUnicodeFont)
-#endif
 #if defined __WXMSW__
   EVT_BUTTON(color_id, Config::OnColorButton)
 #endif
   EVT_LISTBOX(listbox_styleFor, Config::OnChangeStyle)
   EVT_COMBOBOX(language_id, Config::OnChangeWarning)
-  EVT_COMBOBOX(panel_size, Config::OnChangeWarning)
   EVT_CHECKBOX(checkbox_bold, Config::OnCheckbox)
   EVT_CHECKBOX(checkbox_italic, Config::OnCheckbox)
   EVT_CHECKBOX(checkbox_underlined, Config::OnCheckbox)
-  EVT_CHECKBOX(checkbox_greek, Config::OnCheckGreek)
   EVT_BUTTON(save_id, Config::LoadSave)
   EVT_BUTTON(load_id, Config::LoadSave)
   EVT_BUTTON(style_font_family, Config::OnChangeFontFamily)
