@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2006-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2006-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -18,11 +18,14 @@
 ///
 
 #include <wx/clipbrd.h>
+#include <wx/regex.h>
 
 #include "EditorCell.h"
 #include "wxMaxima.h"
 #include "wxMaximaFrame.h"
 
+#define ESC_CHAR wxT('\xA6')
+
 EditorCell::EditorCell() : MathCell()
 {
   m_text = wxEmptyString;
@@ -45,6 +48,7 @@ EditorCell::EditorCell() : MathCell()
   m_saveValue = false;
   m_containsChanges = false;
   m_containsChangesCheck = false;
+  m_firstLineOnly = false;
 }
 
 EditorCell::~EditorCell()
@@ -111,6 +115,9 @@ wxString EditorCell::ToXML(bool all)
     case MC_TYPE_SECTION:
       head += wxT(" type=\"section\"");
       break;
+    case MC_TYPE_SUBSECTION:
+      head += wxT(" type=\"subsection\"");
+      break;
     case MC_TYPE_INPUT:
     default:
       head += wxT(" type=\"input\"");
@@ -159,6 +166,10 @@ void EditorCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
       prevNewLinePos = newLinePos;
     }
 
+    // new
+    if (m_firstLineOnly)
+      m_numberOfLines = 1;
+
     if (m_text == wxEmptyString)
       width = m_charWidth;
 
@@ -196,6 +207,10 @@ void EditorCell::Draw(CellParser& parser, wxPoint point1, int fontsize, bool all
   {
     dc.SetLogicalFunction(wxCOPY); // opaque (for everything except the caret)
 
+    // Need correct m_currentPoint before we call MathCell::Draw!
+    m_currentPoint.x = point.x;
+    m_currentPoint.y = point.y;
+
     if (m_isActive) // draw selection or matching parens
     {
       //
@@ -247,8 +262,7 @@ void EditorCell::Draw(CellParser& parser, wxPoint point1, int fontsize, bool all
         wxRect rect = GetRect(); // rectangle representing the cell
         dc.SetPen(wxNullPen); // no border on rectangles
 #else
-        dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_SELECTION), 1, 1))
-); // window linux, set a pen
+        dc.SetPen(*(wxThePenList->FindOrCreatePen(parser.GetColor(TS_SELECTION), 1, 1))); // window linux, set a pen
 #endif
         dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(parser.GetColor(TS_SELECTION))) ); //highlight c.
 
@@ -278,22 +292,47 @@ void EditorCell::Draw(CellParser& parser, wxPoint point1, int fontsize, bool all
     if (parser.GetChangeAsterisk())  // replace "*" with centerdot for the time of drawing
       m_text.Replace(wxT("*"), wxT("\xB7"));
 #endif
-    while (newLinePos < m_text.Length())
-    {
+    if (!m_firstLineOnly) // draw whole text
       while (newLinePos < m_text.Length())
       {
-        if (m_text.GetChar(newLinePos) == '\n')
-          break;
+        while (newLinePos < m_text.Length())
+        {
+          if (m_text.GetChar(newLinePos) == '\n')
+            break;
+          newLinePos++;
+        }
+
+        dc.DrawText(m_text.SubString(prevNewLinePos, newLinePos - 1),
+            point.x + SCALE_PX(2, scale),
+            point.y - m_center + SCALE_PX(2, scale) + m_charHeight * numberOfLines);
+
         newLinePos++;
+        prevNewLinePos = newLinePos;
+        numberOfLines++;
       }
+    else { // draw only first line (+ some info)
+      wxString firstline;
+      while (newLinePos < m_text.Length())
+      {
+        while (newLinePos < m_text.Length())
+        {
+          if (m_text.GetChar(newLinePos) == '\n')
+            break;
+          newLinePos++;
+        }
 
-      dc.DrawText(m_text.SubString(prevNewLinePos, newLinePos - 1),
-                  point.x + SCALE_PX(2, scale),
-                  point.y - m_center + SCALE_PX(2, scale) + m_charHeight * numberOfLines);
+        if (numberOfLines == 0)
+          firstline = m_text.SubString(0, newLinePos - 1);
 
-      newLinePos++;
-      prevNewLinePos = newLinePos;
-      numberOfLines++;
+        newLinePos++;
+        numberOfLines++;
+      }
+      if (numberOfLines < 1)
+        numberOfLines = 1;
+      firstline << wxT("... (") << numberOfLines - 1 << wxT(" ") << _("lines hidden") << wxT(")");
+      dc.DrawText(firstline,
+          point.x + SCALE_PX(2, scale),
+          point.y - m_center + SCALE_PX(2, scale));
     }
 #if defined __WXMSW__ || wxUSE_UNICODE
     if (parser.GetChangeAsterisk()) // replace centerdot with "*"
@@ -348,13 +387,12 @@ void EditorCell::SetFont(CellParser& parser, int fontsize)
   fontsize1 = MAX(fontsize1, 1);
 
   m_fontName = parser.GetFontName(m_textStyle);
-  m_fontEncoding = parser.GetFontEncoding();
   m_fontStyle = parser.IsItalic(m_textStyle);
   m_fontWeight = parser.IsBold(m_textStyle);
   m_underlined = parser.IsUnderlined(m_textStyle);
   m_fontEncoding = parser.GetFontEncoding();
 
-  dc.SetFont(wxFont(fontsize1, wxMODERN,
+  dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
                     m_fontStyle,
                     m_fontWeight,
                     m_underlined,
@@ -449,7 +487,7 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     else
       m_selectionEnd = m_selectionStart = -1;
 
-    if (m_positionOfCaret < m_text.Length())
+    if (m_positionOfCaret < (signed)m_text.Length())
       m_positionOfCaret++;
 
     if (event.ShiftDown())
@@ -478,7 +516,7 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
       if (line < m_numberOfLines-1) // can we go down ?
         m_positionOfCaret = XYToPosition(column, line + 1);
       else { // we can't go down. move caret to the end
-        m_positionOfCaret = m_text.Length();
+        m_positionOfCaret = (signed)m_text.Length();
         m_caretColumn = -1; // make caretColumn invalid
       }
 
@@ -545,10 +583,10 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
       m_selectionEnd = m_selectionStart = -1;
 
     if (event.ControlDown())
-      m_positionOfCaret = m_text.Length();
+      m_positionOfCaret = (signed)m_text.Length();
     else
     {
-      while (m_positionOfCaret < m_text.Length() &&
+      while (m_positionOfCaret < (signed)m_text.Length() &&
              m_text.GetChar(m_positionOfCaret) != '\n')
         m_positionOfCaret++;
     }
@@ -584,7 +622,7 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
   case WXK_DELETE:
     if (m_selectionStart == -1)
     {
-      if (m_positionOfCaret < m_text.Length())
+      if (m_positionOfCaret < (signed)m_text.Length())
       {
         m_isDirty = true;
         m_containsChanges = true;
@@ -625,41 +663,52 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     {
       m_containsChanges = true;
       m_isDirty = true;
+
+      /// If deleting ( in () then delete both.
+      int right = m_positionOfCaret;
+      if ((m_text.GetChar(m_positionOfCaret-1) == '[' && m_text.GetChar(m_positionOfCaret) == ']') ||
+          (m_text.GetChar(m_positionOfCaret-1) == '(' && m_text.GetChar(m_positionOfCaret) == ')') ||
+          (m_text.GetChar(m_positionOfCaret-1) == '{' && m_text.GetChar(m_positionOfCaret) == '}') ||
+          (m_text.GetChar(m_positionOfCaret-1) == '"' && m_text.GetChar(m_positionOfCaret) == '"'))
+        right++;
       m_text = m_text.SubString(0, m_positionOfCaret - 2) +
-               m_text.SubString(m_positionOfCaret, m_text.Length());
+               m_text.SubString(right, m_text.Length());
       m_positionOfCaret--;
     }
     break;
 
   case WXK_TAB:
     m_isDirty = true;
-    m_containsChanges = true;
+    if (!FindNextTemplate(event.ShiftDown()))
     {
-      if (m_selectionStart > -1) {
-        long start = MIN(m_selectionEnd, m_selectionStart);
-        long end = MAX(m_selectionEnd, m_selectionStart);
-        m_text = m_text.SubString(0, start - 1) +
-                 m_text.SubString(end, m_text.Length());
-        m_positionOfCaret = start;
-        m_selectionEnd = m_selectionStart = -1;
-        break;
+      m_containsChanges = true;
+      {
+        if (m_selectionStart > -1) {
+          long start = MIN(m_selectionEnd, m_selectionStart);
+          long end = MAX(m_selectionEnd, m_selectionStart);
+          m_text = m_text.SubString(0, start - 1) +
+                   m_text.SubString(end, m_text.Length());
+          m_positionOfCaret = start;
+          m_selectionEnd = m_selectionStart = -1;
+          break;
+        }
+
+        int col, line;
+        PositionToXY(m_positionOfCaret, &col, &line);
+        wxString ins;
+        do {
+          col++;
+          ins += wxT(" ");
+        } while (col%4 != 0);
+
+        m_text = m_text.SubString(0, m_positionOfCaret - 1) +
+                 ins +
+                 m_text.SubString(m_positionOfCaret, m_text.Length());
+        m_positionOfCaret += ins.Length();
       }
-
-      int col, line;
-      PositionToXY(m_positionOfCaret, &col, &line);
-      wxString ins;
-      do {
-        col++;
-        ins += wxT(" ");
-      } while (col%4 != 0);
-
-      m_text = m_text.SubString(0, m_positionOfCaret - 1) +
-               ins +
-               m_text.SubString(m_positionOfCaret, m_text.Length());
-      m_positionOfCaret += ins.Length();
     }
     break;
-
+/*
   case WXK_SPACE:
     if (event.ShiftDown())
       m_text = m_text.SubString(0, m_positionOfCaret - 1) + wxT("*") + // wxT("\x00B7")
@@ -671,6 +720,52 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
     m_containsChanges = true;
     m_positionOfCaret++;
     break;
+*/
+  case WXK_ESCAPE:
+    if (m_selectionStart != -1)
+    {
+      m_positionOfCaret = m_selectionEnd;
+      m_selectionStart = m_selectionEnd = -1;
+    }
+#if wxUSE_UNICODE
+    else
+    {
+      // TODO: search only a few positions back for an escchar (10? and not over newlines)
+      bool insertescchar = false;
+      int esccharpos = m_text.Left(m_positionOfCaret).Find(ESC_CHAR, true);
+      if (esccharpos > -1) { // we have a match, check for insertion
+        wxString greek = InterpretEscapeString(m_text.SubString(esccharpos + 1, m_positionOfCaret - 1));
+        if (greek.Length() > 0 ) {
+          m_text = m_text.SubString(0, esccharpos - 1) + greek +
+            m_text.SubString(m_positionOfCaret, m_text.Length());
+          m_positionOfCaret = esccharpos + greek.Length();
+          m_isDirty = true;
+          m_containsChanges = true;
+        }
+        else
+          insertescchar = true;
+      }
+      else
+        insertescchar = true;
+
+      if (insertescchar) {
+        m_text = m_text.SubString(0, m_positionOfCaret - 1) + ESC_CHAR +
+          m_text.SubString(m_positionOfCaret, m_text.Length());
+        m_isDirty = true;
+        m_containsChanges = true;
+        m_positionOfCaret++;
+      }
+    }
+#endif
+    break;
+
+    /* Ignored keys */
+  case WXK_WINDOWS_LEFT:
+  case WXK_WINDOWS_RIGHT:
+  case WXK_WINDOWS_MENU:
+  case WXK_COMMAND:
+  case WXK_START:
+    break;
 
   default:
     if (event.ControlDown())
@@ -775,6 +870,11 @@ void EditorCell::ProcessEvent(wxKeyEvent &event)
                    wxT("}") +
                    m_text.SubString(m_positionOfCaret, m_text.Length());
           break;
+        case '"':
+          m_text = m_text.SubString(0, m_positionOfCaret - 1) +
+                   wxT("\"") +
+                   m_text.SubString(m_positionOfCaret, m_text.Length());
+          break;
         case ')': // jump over ')'
           if (m_text.GetChar(m_positionOfCaret) == ')')
             m_text = m_text.SubString(0, m_positionOfCaret - 2) +
@@ -871,6 +971,173 @@ void EditorCell::FindMatchingParens()
     m_paren1 = m_paren2 = -1;
 }
 
+#if wxUSE_UNICODE
+wxString EditorCell::InterpretEscapeString(wxString txt)
+{
+  long int unicodeval = -1;
+
+  if ((txt == wxT("a")) || (txt == wxT("alpha")))
+    return L"\x03B1";
+  else if ((txt == wxT("b")) || (txt == wxT("beta")))
+    return L"\x03B2";
+  else if ((txt == wxT("g")) || (txt == wxT("gamma")))
+    return L"\x03B3";
+  else if ((txt == wxT("d")) || (txt == wxT("delta")))
+    return L"\x03B4";
+  else if ((txt == wxT("e")) || (txt == wxT("epsilon")))
+    return L"\x03B5";
+  else if ((txt == wxT("z")) || (txt == wxT("zeta")))
+    return L"\x03B6";
+  else if ((txt == wxT("h")) || (txt == wxT("eta")))
+    return L"\x03B7";
+  else if ((txt == wxT("q")) || (txt == wxT("theta")))
+    return L"\x03B8";
+  else if ((txt == wxT("i")) || (txt == wxT("iota")))
+    return L"\x03B9";
+  else if ((txt == wxT("k")) || (txt == wxT("kappa")))
+    return L"\x03BA";
+  else if ((txt == wxT("l")) || (txt == wxT("lambda")))
+    return L"\x03BB";
+  else if ((txt == wxT("m")) || (txt == wxT("mu")))
+    return L"\x03BC";
+  else if ((txt == wxT("n")) || (txt == wxT("nu")))
+    return L"\x03BD";
+  else if ((txt == wxT("x")) || (txt == wxT("xi")))
+    return L"\x03BE";
+  else if ((txt == wxT("om")) || (txt == wxT("omicron")))
+    return L"\x03BF";
+  else if ((txt == wxT("p")) || (txt == wxT("pi")))
+    return L"\x03C0";
+  else if ((txt == wxT("r")) || (txt == wxT("rho")))
+    return L"\x03C1";
+  else if ((txt == wxT("s")) || (txt == wxT("sigma")))
+    return L"\x03C3";
+  else if ((txt == wxT("t")) || (txt == wxT("tau")))
+    return L"\x03C4";
+  else if ((txt == wxT("u")) || (txt == wxT("upsilon")))
+    return L"\x03C5";
+  else if ((txt == wxT("f")) || (txt == wxT("phi")))
+    return L"\x03C6";
+  else if ((txt == wxT("c")) || (txt == wxT("chi")))
+    return L"\x03C7";
+  else if ((txt == wxT("y")) || (txt == wxT("psi")))
+    return L"\x03C8";
+  else if ((txt == wxT("o")) || (txt == wxT("omega")))
+    return L"\x03C9";
+  else if (txt == wxT("Alpha"))
+    return L"\x0391";
+  else if (txt == wxT("Beta"))
+    return L"\x0392";
+  else if (txt == wxT("Gamma"))
+    return L"\x0393";
+  else if (txt == wxT("Delta"))
+    return L"\x0394";
+  else if (txt == wxT("Epsilon"))
+    return L"\x0395";
+  else if (txt == wxT("Zeta"))
+    return L"\x0396";
+  else if (txt == wxT("Eta"))
+    return L"\x0397";
+  else if (txt == wxT("Theta"))
+    return L"\x0398";
+  else if (txt == wxT("Iota"))
+    return L"\x0399";
+  else if (txt == wxT("Kappa"))
+    return L"\x039A";
+  else if (txt == wxT("Lambda"))
+    return L"\x039B";
+  else if (txt == wxT("Mu"))
+    return L"\x039C";
+  else if (txt == wxT("Nu"))
+    return L"\x039D";
+  else if (txt == wxT("Xi"))
+    return L"\x039E";
+  else if (txt == wxT("Omicron"))
+    return L"\x039F";
+  else if (txt == wxT("Pi"))
+    return L"\x03A0";
+  else if (txt == wxT("Rho"))
+    return L"\x03A1";
+  else if (txt == wxT("Sigma"))
+    return L"\x03A3";
+  else if (txt == wxT("Tau"))
+    return L"\x03A4";
+  else if (txt == wxT("Upsilon"))
+    return L"\x03A5";
+  else if (txt == wxT("Phi"))
+    return L"\x03A6";
+  else if (txt == wxT("Chi"))
+    return L"\x03A7";
+  else if (txt == wxT("Psi"))
+    return L"\x03A8";
+  else if (txt == wxT("Omega"))
+    return L"\x03A9";
+  //////////////////////////
+  else if (txt == wxT("2"))
+    return L"\x00B2";
+  else if (txt == wxT("3"))
+    return L"\x00B3";
+  else if (txt == wxT("/2"))
+    return L"\x00BD";
+  else if (txt == wxT("sq"))
+    return L"\x221A";
+  else if (txt == wxT("ii"))
+    return L"\x2148";
+  else if (txt == wxT("ee"))
+    return L"\x2147";
+  else if (txt == wxT("hb"))
+    return L"\x210F";
+  else if (txt == wxT("in"))
+    return L"\x2208";
+  else if (txt == wxT("impl"))
+    return L"\x21D2";
+  else if (txt == wxT("inf"))
+    return L"\x221e";
+  else if (txt == wxT("empty"))
+    return L"\x2205";
+  else if (txt == wxT("TB"))
+    return L"\x25b6";
+  else if (txt == wxT("tb"))
+    return L"\x25b8";
+  else if (txt == wxT("and"))
+    return L"\x22C0";
+  else if (txt == wxT("or"))
+    return L"\x22C1";
+  else if (txt == wxT("xor"))
+    return L"\x22BB";
+  else if (txt == wxT("nand"))
+    return L"\x22BC";
+  else if (txt == wxT("nor"))
+    return L"\x22BD";
+  else if (txt == wxT("implies"))
+    return L"\x21D2";
+  else if (txt == wxT("eq"))
+    return L"\x21D4";
+  else if (txt == wxT("not"))
+    return L"\x00AC";
+  else if (txt == wxT("union"))
+    return L"\x22C3";
+  else if (txt == wxT("inter"))
+    return L"\x22C2";
+  else if (txt == wxT("subseteq"))
+    return L"\x2286";
+  else if (txt == wxT("subset"))
+    return L"\x2282";
+  else if (txt == wxT("notsubseteq"))
+    return L"\x2288";
+  else if (txt == wxT("notsubset"))
+    return L"\x2284";
+
+  /////////////////////////
+  else if (txt.ToLong(&unicodeval, 16))
+    return wxString::Format(wxT("%c"), unicodeval);
+
+  /////////////////////////
+  else
+    return wxEmptyString;
+}
+#endif
+
 bool EditorCell::ActivateCell()
 {
   m_isActive = !m_isActive;
@@ -882,6 +1149,14 @@ bool EditorCell::ActivateCell()
   m_selectionEnd = m_selectionStart = -1;
   m_paren1 = m_paren2 = -1;
 
+  // upon activation unhide the parent groupcell
+  if (m_isActive) {
+    m_firstLineOnly = false;
+    ((GroupCell *)GetParent())->Hide(false);
+    if (GetType() == MC_TYPE_INPUT)
+      FindMatchingParens();
+  }
+
   return true;
 }
 
@@ -890,7 +1165,7 @@ bool EditorCell::AddEnding()
   wxString text = m_text.Trim();
   if (text.Right(1) != wxT(";") && text.Right(1) != wxT("$")) {
     m_text += wxT(";");
-    m_width = -1;
+    m_paren1 = m_paren2 = m_width = -1;
     return true;
   }
   return false;
@@ -995,7 +1270,7 @@ void EditorCell::SelectPointText(wxDC& dc, wxPoint& point)
   int lineStart = XYToPosition(0, lin);
   m_positionOfCaret = lineStart;
 
-  while (m_text.GetChar(m_positionOfCaret) != '\n' && m_positionOfCaret < m_text.Length())
+  while (m_text.GetChar(m_positionOfCaret) != '\n' && m_positionOfCaret < (signed)m_text.Length())
   {
     s = m_text.SubString(lineStart, m_positionOfCaret);
     dc.GetTextExtent(m_text.SubString(lineStart, m_positionOfCaret),
@@ -1006,7 +1281,7 @@ void EditorCell::SelectPointText(wxDC& dc, wxPoint& point)
     m_positionOfCaret++;
   }
 
-  m_positionOfCaret = MIN(m_positionOfCaret, m_text.Length());
+  m_positionOfCaret = MIN(m_positionOfCaret, (signed)m_text.Length());
 
   m_displayCaret = true;
   m_caretColumn = -1;
@@ -1058,7 +1333,7 @@ bool EditorCell::IsPointInSelection(wxDC& dc, wxPoint point)
   int width, height;
   int lineStart = XYToPosition(0, lin);
   int positionOfCaret = lineStart;
-  while (m_text.GetChar(positionOfCaret) != '\n' && positionOfCaret < m_text.Length())
+  while (m_text.GetChar(positionOfCaret) != '\n' && positionOfCaret < (signed)m_text.Length())
   {
     s = m_text.SubString(lineStart, positionOfCaret);
     dc.GetTextExtent(m_text.SubString(lineStart, positionOfCaret),
@@ -1067,45 +1342,77 @@ bool EditorCell::IsPointInSelection(wxDC& dc, wxPoint point)
       break;
     positionOfCaret++;
   }
-  positionOfCaret = MIN(positionOfCaret, m_text.Length());
+  positionOfCaret = MIN(positionOfCaret, (signed)m_text.Length());
 
   if ((m_selectionStart >= positionOfCaret) || (m_selectionEnd <= positionOfCaret))
     return false;
 
   return true;
 }
+
+// DivideAtCaret
+// returns the string from caret to end and
+// modifies the m_text so it contains only the string
+// from beginning to caret
+// Used for 'Divide Cell', called from MathCtrl
+wxString EditorCell::DivideAtCaret()
+{
+  wxString original = m_text;
+  m_containsChanges = true;
+  m_text = m_text.SubString(0, m_positionOfCaret - 1);
+  ResetSize();
+  GetParent()->ResetSize();
+  return original.SubString(m_positionOfCaret, original.Length());
+}
+
+void EditorCell::CommentSelection()
+{
+  if ((m_selectionStart == -1) || (m_selectionEnd == -1))
+    return;
+  m_containsChanges = true;
+  m_isDirty = true;
+  m_text = m_text.SubString(0, m_selectionStart - 1) + wxT("/*")
+    + m_text.SubString(m_selectionStart, m_selectionEnd - 1) + wxT("*/")
+    + m_text.SubString(m_selectionEnd, m_text.Length());
+  m_positionOfCaret = MIN(m_selectionEnd + 4, (signed)m_text.Length());
+  m_selectionStart = m_selectionEnd = -1;
+}
+
 /***
  * SelectWordUnderCaret
- * - called from MathCtrl::OnDoubleClick and wxMaxima::HelpMenu
- * Selects word under cursor (aA-zZ, 0-9 counts) or
- * the inside of brackets using m_paren1 and m_paren2 if available.
- * Returns the selected string if selected a word succesfully - used for F1 help.
+ * - called from MathCtrl::OnDoubleClick, MathCtrl::Autocomplete and wxMaxima::HelpMenu
+ * Selects word under cursor (aA-zZ, 0-9, %, _, count) or
+ * the inside of brackets using m_paren1 and m_paren2 if available and selectParens is true.
+ * Returns the selected string if selected a word succesfully - used for F1 help and
+ * MathCtrl::Autocomplete.
  */
-wxString EditorCell::SelectWordUnderCaret()
+
+wxString EditorCell::SelectWordUnderCaret(bool selectParens, bool toRight)
 {
-  if ((m_paren1 != -1) && (m_paren2 != -1)) {
+  if (selectParens && (m_paren1 != -1) && (m_paren2 != -1)) {
     m_selectionStart = MIN(m_paren1,m_paren2) + 1;
     m_selectionEnd = MAX(m_paren1, m_paren2);
     m_positionOfCaret = m_selectionEnd;
     return wxT("%");
   }
-  wxString wordChars = wxT("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_");
+  wxString wordChars = wxT("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890_%");
 
   long left = m_positionOfCaret, right = m_positionOfCaret;
   while (left > 0)
   {
-    if(wordChars.Find(m_text.GetChar(left)) == -1) {
-      left++;
+    if (wordChars.Find(m_text.GetChar(left-1)) == -1)
       break;
-    }
     left--;
   }
 
-  while (right < m_text.length() )
+  if (toRight)
   {
-    if(wordChars.Find(m_text.GetChar(right)) == -1)
-      break;
-    right++;
+    while (right < (signed)m_text.length() )
+    {
+      if(wordChars.Find(m_text.GetChar(right)) == -1)
+        break;
+      right++;
+    }
   }
 
   if (left != right)
@@ -1125,6 +1432,7 @@ bool EditorCell::CopyToClipboard()
     return false;
   if (wxTheClipboard->Open())
   {
+    wxTheClipboard->UsePrimarySelection(false);
     long start = MIN(m_selectionStart, m_selectionEnd);
     long end = MAX(m_selectionStart, m_selectionEnd) - 1;
     wxString s = m_text.SubString(start, end);
@@ -1158,10 +1466,11 @@ bool EditorCell::CutToClipboard()
   return true;
 }
 
-void EditorCell::PasteFromClipboard()
+void EditorCell::PasteFromClipboard(bool primary)
 {
   if (wxTheClipboard->Open())
   {
+    wxTheClipboard->UsePrimarySelection(primary);
     if (wxTheClipboard->IsSupported(wxDF_TEXT))
     {
       wxTextDataObject obj;
@@ -1231,6 +1540,40 @@ void EditorCell::SaveValue()
   m_oldEnd = m_selectionEnd;
 }
 
+void EditorCell::SetValue(wxString text)
+{
+  if (m_type == MC_TYPE_INPUT && m_matchParens)
+  {
+    if (text == wxT("(")) {
+      m_text = wxT("()");
+      m_positionOfCaret = 1;
+    }
+    else if (text == wxT("[")) {
+      m_text = wxT("[]");
+      m_positionOfCaret = 1;
+    }
+    else if (text == wxT("{")) {
+      m_text = wxT("{}");
+      m_positionOfCaret = 1;
+    }
+    else if (text == wxT("\"")) {
+      m_text = wxT("\"\"");
+      m_positionOfCaret = 1;
+    }
+    else {
+      m_text = text;
+      m_positionOfCaret = m_text.Length();
+    }
+  }
+  else
+  {
+    m_text = text;
+    m_positionOfCaret = m_text.Length();
+  }
+
+  m_containsChanges = true;
+}
+
 bool EditorCell::CheckChanges()
 {
   if (m_containsChanges != m_containsChangesCheck) {
@@ -1240,3 +1583,167 @@ bool EditorCell::CheckChanges()
 
   return false;
 }
+
+int EditorCell::ReplaceAll(wxString oldString, wxString newString)
+{
+  SaveValue();
+  int count = m_text.Replace(oldString, newString);
+  if (count > 0)
+  {
+    m_containsChanges = true;
+    m_selectionStart = m_selectionEnd = -1;
+  }
+  return count;
+}
+
+bool EditorCell::FindNext(wxString str, bool down, bool ignoreCase)
+{
+  int start = down ? 0 : m_text.Length();
+  wxString text(m_text);
+
+  if (ignoreCase)
+  {
+    str.MakeLower();
+    text.MakeLower();
+  }
+
+  if (m_selectionStart >= 0)
+  {
+    if (down)
+      start = m_selectionStart + 1;
+    else
+      start = m_selectionStart - 1;
+  }
+  else if (m_isActive)
+    start = m_positionOfCaret;
+
+  if (!down && m_selectionStart == 0)
+    return false;
+
+  int strStart = wxNOT_FOUND;
+  if (down)
+    strStart = text.find(str, start);
+  else
+    strStart = text.rfind(str, start);
+
+  if (strStart != wxNOT_FOUND)
+  {
+    m_selectionStart = strStart;
+    m_selectionEnd = strStart + str.Length();
+    return true;
+  }
+  return false;
+}
+
+bool EditorCell::ReplaceSelection(wxString oldStr, wxString newStr)
+{
+  if (m_selectionStart > -1 &&
+      m_text.SubString(m_selectionStart, m_selectionEnd - 1) == oldStr)
+  {
+    m_text = m_text.SubString(0, m_selectionStart - 1) +
+             newStr +
+             m_text.SubString(m_selectionEnd, m_text.Length());
+    m_containsChanges = -1;
+    m_positionOfCaret = m_selectionEnd = m_selectionStart + newStr.Length();
+
+    if (GetType() == MC_TYPE_INPUT)
+      FindMatchingParens();
+
+    return true;
+  }
+  return false;
+}
+
+wxString EditorCell::GetSelectionString()
+{
+  if (m_selectionStart == -1 || m_selectionEnd == -1)
+    return wxEmptyString;
+  return m_text.SubString(m_selectionStart, m_selectionEnd-1);
+}
+
+void EditorCell::ClearSelection()
+{
+  if (m_selectionStart == -1 || m_selectionEnd == -1)
+    return;
+
+  m_positionOfCaret = m_selectionEnd;
+  m_selectionStart = m_selectionEnd = -1;
+}
+
+bool EditorCell::FindNextTemplate(bool left)
+{
+  wxRegEx varsRegex;
+
+  if (left)
+    varsRegex.Compile(wxT("(<[^> \n]+>)[^>]*$"));
+  else
+    varsRegex.Compile(wxT("(<[^> \n]+>)"));
+
+  int positionOfCaret = m_positionOfCaret;
+  if (!left && m_selectionEnd != -1)
+    positionOfCaret = m_selectionEnd;
+
+
+  // Splits the string into first (from caret in the direction of search)
+  // and second (the rest of the string)
+  wxString first, second;
+  if (left)
+  {
+    first = m_text.Mid(0, positionOfCaret);
+    second = m_text.Mid(positionOfCaret);
+  }
+  else
+  {
+    first = m_text.Mid(positionOfCaret);
+    second = m_text.Mid(0, positionOfCaret);
+  }
+
+  size_t start, length;
+
+  // First search in the direction of search
+  if (varsRegex.Matches(first))
+  {
+    varsRegex.GetMatch(&start, &length, 1);
+    if (left)
+      m_positionOfCaret = m_selectionStart = start;
+    else
+      m_positionOfCaret = m_selectionStart = positionOfCaret + start;
+    m_selectionEnd = m_selectionStart + length;
+    return true;
+  }
+
+  // Then in the rest of the string
+  if (varsRegex.Matches(second))
+  {
+    varsRegex.GetMatch(&start, &length, 1);
+    if (!left)
+      m_positionOfCaret = m_selectionStart = start;
+    else
+      m_positionOfCaret = m_selectionStart = positionOfCaret + start;
+    m_selectionEnd = m_selectionStart + length;
+    return true;
+  }
+
+  return false;
+}
+
+void EditorCell::CaretToEnd()
+{
+  m_positionOfCaret = m_text.Length();
+  if (GetType() == MC_TYPE_INPUT)
+    FindMatchingParens();
+}
+
+void EditorCell::CaretToStart()
+{
+  m_positionOfCaret = 0;
+  if (GetType() == MC_TYPE_INPUT)
+    FindMatchingParens();
+}
+
+void EditorCell::CaretToPosition(int pos)
+{
+  m_positionOfCaret = pos;
+  if (GetType() == MC_TYPE_INPUT)
+    FindMatchingParens();
+}
