@@ -1,5 +1,6 @@
 ///
-///  Copyright (C) 2008 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///            (C) 2008-2009 Ziga Lenarcic <zigalenarcic@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -17,6 +18,9 @@
 ///  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ///
 
+#include <wx/config.h>
+#include <wx/clipbrd.h>
+
 #include "GroupCell.h"
 #include "TextCell.h"
 #include "EditorCell.h"
@@ -27,6 +31,7 @@ GroupCell::GroupCell(int groupType, wxString initString) : MathCell()
 {
   m_input = NULL;
   m_output = NULL;
+  m_hiddenTree = NULL;
   m_outputRect.x = -1;
   m_outputRect.y = -1;
   m_outputRect.width = 0;
@@ -40,14 +45,22 @@ GroupCell::GroupCell(int groupType, wxString initString) : MathCell()
   m_working = false;
   m_groupType = groupType;
   m_lastInOutput = NULL;
+  m_appendedCells = NULL;
+
   // set up cell depending on groupType, so we have a working cell
-  if (groupType == GC_TYPE_CODE)
-    m_input = new TextCell(wxT(">> "));
-  else
-    m_input = new TextCell(wxEmptyString);
-  m_input->SetType(MC_TYPE_MAIN_PROMPT);
+  if (groupType != GC_TYPE_PAGEBREAK) {
+    if (groupType == GC_TYPE_CODE)
+      m_input = new TextCell(EMPTY_INPUT_LABEL);
+    else
+      m_input = new TextCell(wxT(" "));
+
+    m_input->SetType(MC_TYPE_MAIN_PROMPT);
+  }
+
+  bool match = true;
+  wxConfig::Get()->Read(wxT("matchParens"), &match);
   EditorCell *editor = new EditorCell();
-  editor->SetValue(initString);
+  editor->SetMatchParens(match);
 
   switch (groupType) {
     case GC_TYPE_CODE:
@@ -55,22 +68,40 @@ GroupCell::GroupCell(int groupType, wxString initString) : MathCell()
       AppendInput(editor);
       break;
     case GC_TYPE_TEXT:
+      m_input->SetType(MC_TYPE_TEXT);
       editor->SetType(MC_TYPE_TEXT);
-      AppendOutput(editor);
+      AppendInput(editor);
       break;
     case GC_TYPE_TITLE:
+      m_input->SetType(MC_TYPE_TITLE);
       editor->SetType(MC_TYPE_TITLE);
-      AppendOutput(editor);
+      AppendInput(editor);
       break;
     case GC_TYPE_SECTION:
+      m_input->SetType(MC_TYPE_SECTION);
       editor->SetType(MC_TYPE_SECTION);
-      AppendOutput(editor);
+      AppendInput(editor);
+      break;
+    case GC_TYPE_SUBSECTION:
+      m_input->SetType(MC_TYPE_SUBSECTION);
+      editor->SetType(MC_TYPE_SUBSECTION);
+      AppendInput(editor);
       break;
     case GC_TYPE_IMAGE:
+      m_input->SetType(MC_TYPE_TEXT);
+      editor->SetType(MC_TYPE_TEXT);
+      editor->SetValue(wxEmptyString);
+      AppendInput(editor);
+      break;
     default:
-      delete(editor);
+      delete editor;
+      editor = NULL;
       break;
   }
+
+  if (editor != NULL)
+    editor->SetValue(initString);
+
   // when creating an image cell, if a string is provided
   // it loads an image (without deleting it)
   if ((groupType == GC_TYPE_IMAGE) && (initString.Length() > 0)) {
@@ -86,6 +117,8 @@ GroupCell::~GroupCell()
   if (m_input != NULL)
     delete m_input;
   DestroyOutput();
+  if (m_hiddenTree)
+    delete m_hiddenTree;
 }
 
 void GroupCell::SetParent(MathCell *parent, bool all)
@@ -112,12 +145,30 @@ void GroupCell::DestroyOutput()
   m_output = NULL;
 }
 
+// when all=false (default) only reset input label of the current (code) cell
+// if all = true, then also reset next cells and folded cells
+void GroupCell::ResetInputLabel(bool all)
+{
+  if (m_groupType == GC_TYPE_CODE) {
+    if (m_input)
+      m_input->SetValue(EMPTY_INPUT_LABEL);
+  }
+  // if all, also reset input labels in the folded cells
+  else if (all && IsFoldable() && m_hiddenTree)
+    m_hiddenTree->ResetInputLabel(true);
+
+  // reset the next cell
+  if (all && m_next)
+    ((GroupCell *)m_next)->ResetInputLabel(true);
+}
+
 MathCell* GroupCell::Copy(bool all)
 {
   GroupCell* tmp = new GroupCell(m_groupType);
   tmp->Hide(m_hide);
   CopyData(this, tmp);
-  tmp->SetInput(m_input->Copy(true));
+  if (m_input)
+    tmp->SetInput(m_input->Copy(true));
   if (m_output != NULL)
     tmp->SetOutput(m_output->Copy(true));
   if (all && m_next != NULL)
@@ -178,8 +229,11 @@ void GroupCell::SetOutput(MathCell *output)
   m_output->m_group = this;
 
   m_lastInOutput = m_output;
+
   while (m_lastInOutput->m_next != NULL)
     m_lastInOutput = m_lastInOutput->m_next;
+
+  //m_appendedCells = output;
 }
 
 void GroupCell::RemoveOutput()
@@ -187,36 +241,21 @@ void GroupCell::RemoveOutput()
   DestroyOutput();
   ResetSize();
   m_output = NULL;
+  m_lastInOutput = NULL;
+  m_appendedCells = NULL;
   m_hide = false;
 }
 
 void GroupCell::AppendOutput(MathCell *cell)
 {
-  ResetSize();
-
   if (m_output == NULL) {
     m_output = cell;
-    // If there is no output yet and we add text/section/title/image cell
-    // we set m_groupType. This is to support loading wxmx files.
-    switch (cell->GetType()) {
-      case MC_TYPE_TEXT:
-        m_groupType = GC_TYPE_TEXT;
-        break;
-      case MC_TYPE_SECTION:
-        m_groupType = GC_TYPE_SECTION;
-        break;
-      case MC_TYPE_TITLE:
-        m_groupType = GC_TYPE_TITLE;
-        break;
-      case MC_TYPE_IMAGE: // Images produced with plot commands will have labels!
-        m_groupType = GC_TYPE_IMAGE;
-        break;
-    }
 
     if (m_groupType == GC_TYPE_CODE && m_input->m_next != NULL)
       ((EditorCell *)(m_input->m_next))->ContainsChanges(false);
 
     m_lastInOutput = m_output;
+
     while (m_lastInOutput->m_next != NULL)
       m_lastInOutput = m_lastInOutput->m_next;
 
@@ -229,14 +268,38 @@ void GroupCell::AppendOutput(MathCell *cell)
 
     while (tmp->m_next != NULL)
       tmp = tmp->m_next;
+
     tmp->AppendCell(cell);
+
+    while (m_lastInOutput->m_next != NULL)
+      m_lastInOutput = m_lastInOutput->m_next;
   }
+
+  if (m_appendedCells == NULL)
+    m_appendedCells = cell;
+}
+
+void GroupCell::Recalculate(CellParser& parser, int d_fontsize, int m_fontsize)
+{
+  m_fontSize = d_fontsize;
+  m_mathFontSize = m_fontsize;
+
+  RecalculateWidths(parser, d_fontsize, false);
+  RecalculateSize(parser, d_fontsize, false);
 }
 
 void GroupCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
 {
   if (m_width == -1 || m_height == -1 || parser.ForceUpdate())
   {
+    // special case of 'line cell'
+    if (m_groupType == GC_TYPE_PAGEBREAK) {
+      m_width = 10;
+      m_height = 2;
+      MathCell::RecalculateWidths(parser, fontsize, all);
+      return;
+    }
+
     UnBreakUpCells();
 
     double scale = parser.GetScale();
@@ -254,23 +317,33 @@ void GroupCell::RecalculateWidths(CellParser& parser, int fontsize, bool all)
     else {
       MathCell *tmp = m_output;
       while (tmp != NULL) {
-        tmp->RecalculateWidths(parser, fontsize, false);
+        tmp->RecalculateWidths(parser, tmp->IsMath() ? m_mathFontSize : m_fontSize, false);
         tmp = tmp->m_next;
       }
       // This is not correct, m_width will be computed correctly in RecalculateSize!
       m_width = m_input->GetFullWidth(scale);
     }
 
-    BreakUpCells(parser, fontsize, parser.GetClientWidth());
+    BreakUpCells(parser, m_fontSize, parser.GetClientWidth());
     BreakLines(parser.GetClientWidth());
   }
-  MathCell::RecalculateWidths(parser, fontsize, all);
+  MathCell::RecalculateWidths(parser, m_fontSize, all);
 }
 
 void GroupCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
 {
   if (m_width == -1 || m_height == -1 || parser.ForceUpdate())
   {
+    // special case
+    if (m_groupType == GC_TYPE_PAGEBREAK) {
+      m_width = 10;
+      m_height = 2;
+      m_center = 0;
+      m_indent = 0;
+      MathCell::RecalculateWidths(parser, fontsize, all);
+      return;
+    }
+
     double scale = parser.GetScale();
     m_input->RecalculateSize(parser, fontsize, true);
     m_center = m_input->GetMaxCenter();
@@ -280,7 +353,7 @@ void GroupCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
     if (m_output != NULL && !m_hide) {
       MathCell *tmp = m_output;
       while (tmp != NULL) {
-        tmp->RecalculateSize(parser, fontsize, false);
+        tmp->RecalculateSize(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize, false);
         tmp = tmp->m_next;
       }
 
@@ -309,6 +382,50 @@ void GroupCell::RecalculateSize(CellParser& parser, int fontsize, bool all)
   MathCell::RecalculateSize(parser, fontsize, all);
 }
 
+// We assume that appended cells will be in a new line!
+void GroupCell::RecalculateAppended(CellParser& parser)
+{
+  if (m_appendedCells == NULL)
+    return;
+
+  MathCell *tmp = m_appendedCells;
+  int fontsize = m_fontSize;
+  double scale = parser.GetScale();
+
+  // Recalculate widths of cells
+  while (tmp != NULL) {
+    tmp->RecalculateWidths(parser, tmp->IsMath() ? m_mathFontSize : m_fontSize, false);
+    tmp = tmp->m_next;
+  }
+
+  // Breakup cells and break lines
+  BreakUpCells(m_appendedCells, parser, fontsize, parser.GetClientWidth());
+  BreakLines(m_appendedCells, parser.GetClientWidth());
+
+  // Recalculate size of cells
+  tmp = m_appendedCells;
+  while (tmp != NULL) {
+    tmp->RecalculateSize(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize, false);
+    tmp = tmp->m_next;
+  }
+
+  // Update widths
+  tmp = m_appendedCells;
+  while (tmp != NULL) {
+    if (tmp->BreakLineHere() || tmp == m_appendedCells) {
+      m_width = MAX(m_width, tmp->GetLineWidth(scale));
+      m_outputRect.width = MAX(m_outputRect.width, tmp->GetLineWidth(scale));
+      m_height += tmp->GetMaxHeight();
+      if (tmp->m_bigSkip)
+        m_height += MC_LINE_SKIP;
+      m_outputRect.height += tmp->GetMaxHeight() + MC_LINE_SKIP;
+    }
+    tmp = tmp->m_nextToDraw;
+  }
+
+  m_appendedCells = NULL;
+}
+
 void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
 {
   double scale = parser.GetScale();
@@ -319,11 +436,22 @@ void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
   }
   if (DrawThisCell(parser, point))
   {
+    // draw a thick line for 'page break'
+    // and return
+    if (m_groupType == GC_TYPE_PAGEBREAK) {
+      wxRect rect = GetRect(false);
+      int y = rect.GetY();
+      wxPen pen(parser.GetColor(TS_CURSOR), 1, wxDOT);
+      dc.SetPen(pen);
+      dc.DrawLine(0, y , 10000, y);
+      MathCell::Draw(parser, point, fontsize, all);
+      return;
+    }
+
     //
-    // Paint background if we have section, title, text
+    // Paint background if we have a text cell
     //
-    MathCell * editor = GetEditable();
-    if (editor != NULL && editor->GetType() != MC_TYPE_INPUT) {
+    if (m_groupType == GC_TYPE_TEXT) {
       wxRect rect = GetRect(false);
       int y = rect.GetY();
 
@@ -358,7 +486,7 @@ void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
           tmp->m_currentPoint.x = in.x;
           tmp->m_currentPoint.y = in.y;
           if (tmp->DrawThisCell(parser, in))
-            tmp->Draw(parser, in, MAX(fontsize, MC_MIN_SIZE), false);
+            tmp->Draw(parser, in, MAX(tmp->IsMath() ? m_mathFontSize : m_fontSize, MC_MIN_SIZE), false);
           if (tmp->m_nextToDraw != NULL) {
             if (tmp->m_nextToDraw->BreakLineHere()) {
               in.x = m_indent;
@@ -395,32 +523,47 @@ void GroupCell::Draw(CellParser& parser, wxPoint point, int fontsize, bool all)
       dc.SetBrush( *(wxTheBrushList->FindOrCreateBrush(parser.GetColor(TS_CELL_BRACKET)))); //highlight c.
     }
 
-    if (!m_hide) {
+    if ((!m_hide) && (!m_hiddenTree)) {
       dc.SetBrush(*wxTRANSPARENT_BRUSH);
     }
 
-    wxPoint * points = new wxPoint[3];
-    points[0].x = point.x - SCALE_PX(10, scale);
-    points[0].y = point.y - m_center;
-    points[1].x = point.x - SCALE_PX(10, scale);
-    points[1].y = point.y - m_center + SCALE_PX(10, scale);
-    points[2].x = point.x;
-    points[2].y = point.y - m_center;
-    dc.DrawPolygon(3, points);
-    delete(points);
+    if (IsFoldable()) { // draw a square
+      wxPoint *points = new wxPoint[4];
+      points[0].x = point.x - SCALE_PX(10, scale);
+      points[0].y = point.y - m_center;
+      points[1].x = point.x - SCALE_PX(10, scale);
+      points[1].y = point.y - m_center + SCALE_PX(10, scale);
+      points[2].x = point.x;
+      points[2].y = point.y - m_center + SCALE_PX(10, scale);
+      points[3].x = point.x;
+      points[3].y = point.y - m_center;
+      dc.DrawPolygon(4, points);
+      delete(points);
+    }
+    else { // draw a triangle and line
+      wxPoint *points = new wxPoint[3];
+      points[0].x = point.x - SCALE_PX(10, scale);
+      points[0].y = point.y - m_center;
+      points[1].x = point.x - SCALE_PX(10, scale);
+      points[1].y = point.y - m_center + SCALE_PX(10, scale);
+      points[2].x = point.x;
+      points[2].y = point.y - m_center;
+      dc.DrawPolygon(3, points);
+      delete(points);
 
-    // vertical
-    dc.DrawLine(point.x - SCALE_PX(10, scale), point.y - m_center,
-                point.x - SCALE_PX(10, scale), point.y - m_center + m_height);
-    // bottom horizontal
-    dc.DrawLine(point.x - SCALE_PX(10, scale), point.y - m_center + m_height,
-                point.x - SCALE_PX(2, scale) , point.y - m_center + m_height);
-    // middle horizontal
-    if (m_groupType == GC_TYPE_CODE && m_output != NULL && !m_hide) {
-      dc.DrawLine(point.x - SCALE_PX(6, scale),
-                  point.y - m_center + m_input->GetMaxHeight(),
-                  point.x - SCALE_PX(10, scale),
-                  point.y - m_center + m_input->GetMaxHeight());
+      // vertical
+      dc.DrawLine(point.x - SCALE_PX(10, scale), point.y - m_center,
+          point.x - SCALE_PX(10, scale), point.y - m_center + m_height);
+      // bottom horizontal
+      dc.DrawLine(point.x - SCALE_PX(10, scale), point.y - m_center + m_height,
+          point.x - SCALE_PX(2, scale) , point.y - m_center + m_height);
+      // middle horizontal
+      if (m_groupType == GC_TYPE_CODE && m_output != NULL && !m_hide) {
+        dc.DrawLine(point.x - SCALE_PX(6, scale),
+            point.y - m_center + m_input->GetMaxHeight(),
+            point.x - SCALE_PX(10, scale),
+            point.y - m_center + m_input->GetMaxHeight());
+      }
     }
 
     UnsetPen(parser);
@@ -435,78 +578,138 @@ wxRect GroupCell::HideRect()
 
 wxString GroupCell::ToString(bool all)
 {
-  wxString str = m_input->ToString(true);
-  if (m_output != NULL && !m_hide) {
-    MathCell *tmp = m_output;
-    while (tmp != NULL) {
-      if (tmp->ForceBreakLineHere() && str.Length()>0)
-        str += wxT("\n");
-      str += tmp->ToString(false);
-      tmp = tmp->m_nextToDraw;
+  wxString str;
+  if (GetEditable()) {
+    str = m_input->ToString(true);
+    if (m_output != NULL && !m_hide) {
+      MathCell *tmp = m_output;
+      while (tmp != NULL) {
+        if (tmp->ForceBreakLineHere() && str.Length()>0)
+          str += wxT("\n");
+        str += tmp->ToString(false);
+        tmp = tmp->m_nextToDraw;
+      }
     }
   }
   return str + MathCell::ToString(all);
 }
 
+wxString GroupCell::ToTeX(bool all)
+{
+  return ToTeX(all, wxEmptyString, wxEmptyString, NULL);
+}
+
 wxString GroupCell::ToTeX(bool all, wxString imgDir, wxString filename, int *imgCounter)
 {
   wxString str;
-  if (m_groupType == GC_TYPE_CODE) {
-    str = wxT("\n\\begin{verbatim}\n") + m_input->ToString(true) + wxT("\n\\end{verbatim}\n");
+
+  // pagebreak
+  if (m_groupType == GC_TYPE_PAGEBREAK) {
+    str = wxT("\\pagebreak\n");
   }
-  if (m_output != NULL && !m_hide) {
-    if (m_groupType != GC_TYPE_CODE) {
-      str = m_output->ToString(true);
-      switch (m_output->GetStyle()) {
-        case TS_TITLE:
-          str = wxT("\n\\section{") + str + wxT("}\n");
-          break;
-        case TS_SECTION:
-          str = wxT("\n\\subsection{") + str + wxT("}\n");
-          break;
-        default:
-          if (!str.StartsWith(wxT("TeX:")))
-            str = wxT("\n\\begin{verbatim}\n") + str + wxT("\n\\end{verbatim}\n");
-          else
-            str = wxT("\n") + str.SubString(5, str.Length()-1) + wxT("\n");
-          break;
-      }
+
+  // IMAGE CELLS
+  else if (m_groupType == GC_TYPE_IMAGE && imgDir != wxEmptyString) {
+    MathCell *copy = m_output->Copy(false);
+    (*imgCounter)++;
+    wxString image = filename + wxString::Format(wxT("_%d.png"), *imgCounter);
+    wxString file = imgDir + wxT("/") + image;
+
+    Bitmap bmp;
+    bmp.SetData(copy);
+
+    if (!wxDirExists(imgDir))
+      wxMkdir(imgDir);
+
+    if (bmp.ToFile(file))
+    {
+      str << wxT("\\begin{figure}[htb]\n")
+          << wxT("  \\begin{center}\n")
+          << wxT("    \\includegraphics{")
+          << filename << wxT("_img/") << image << wxT("}\n")
+          << wxT("  \\caption{") << m_input->m_next->GetValue() << wxT("}\n")
+          << wxT("  \\end{center}\n")
+          << wxT("\\end{figure}");
     }
-    else {
+  }
+  else if (m_groupType == GC_TYPE_IMAGE)
+    str << wxT("\n\\vert|<<GRAPHICS>>|\n");
+
+  // CODE CELLS
+  else if (m_groupType == GC_TYPE_CODE) {
+    str = wxT("\n\\begin{verbatim}\n") + m_input->ToString(true) + wxT("\n\\end{verbatim}\n");
+
+    if (m_output != NULL) {
       str += wxT("$$\n");
       wxString label;
       MathCell *tmp = m_output;
+
       while (tmp != NULL) {
-        if (tmp->GetType() == MC_TYPE_IMAGE || tmp->GetType() == MC_TYPE_SLIDE) {
-          MathCell *copy = tmp->Copy(false);
-          (*imgCounter)++;
-          wxString image = filename + wxString::Format(wxT("_%d.png"), *imgCounter);
-          wxString file = imgDir + wxT("/") + image;
 
-          Bitmap bmp;
-          bmp.SetData(copy);
+        if (tmp->GetType() == MC_TYPE_IMAGE || tmp->GetType() == MC_TYPE_SLIDE)
+        {
+          if (imgDir != wxEmptyString)
+          {
+            MathCell *copy = tmp->Copy(false);
+            (*imgCounter)++;
+            wxString image = filename + wxString::Format(wxT("_%d.png"), *imgCounter);
+            wxString file = imgDir + wxT("/") + image;
 
-          if (!wxDirExists(imgDir))
-            if (!wxMkdir(imgDir))
-              continue;
+            Bitmap bmp;
+            bmp.SetData(copy);
 
-          if (bmp.ToFile(file))
-            str += wxT("\\includegraphics[width=9cm]{") +
-                filename + wxT("_img/") + image + wxT("}\n");
+            if (!wxDirExists(imgDir))
+              if (!wxMkdir(imgDir))
+                continue;
+
+            if (bmp.ToFile(file))
+              str += wxT("\\includegraphics[width=9cm]{") +
+                  filename + wxT("_img/") + image + wxT("}\n");
+          }
+          else
+            str << wxT("\n\\verb|<<GRAPHICS>>|\n");
         }
+
         else if (tmp->GetStyle() == TS_LABEL)
         {
           if (str.Right(3) != wxT("$$\n"))
             str += label + wxT("\n$$\n$$\n");
           label = wxT("\\leqno{\\tt ") + tmp->ToTeX(false) + wxT(" }");
         }
+
         else
           str += tmp->ToTeX(false);
+
         tmp = tmp->m_nextToDraw;
       }
       str += label + wxT("\n$$\n");
     }
   }
+
+  // TITLES, SECTIONS, SUBSECTIONS, TEXT
+  else if (GetEditable() != NULL && !m_hide) {
+    str = GetEditable()->ToTeX(true);
+    switch (GetEditable()->GetStyle()) {
+      case TS_TITLE:
+        str = wxT("\n\\pagebreak{}\n{\\Huge {\\sc ") + str + wxT("}}\n");
+        str += wxT("\\setcounter{section}{0}\n\\setcounter{subsection}{0}\n");
+        str += wxT("\\setcounter{figure}{0}\n");
+        break;
+      case TS_SECTION:
+        str = wxT("\n\\section{") + str + wxT("}\n");
+        break;
+      case TS_SUBSECTION:
+        str = wxT("\n\\subsection{") + str + wxT("}\n");
+        break;
+      default:
+        if (!str.StartsWith(wxT("TeX:")))
+          str = wxT("\n\\begin{verbatim}\n") + str + wxT("\n\\end{verbatim}\n");
+        else
+          str = wxT("\n") + str.SubString(5, str.Length()-1) + wxT("\n");
+        break;
+    }
+  }
+
   return str + MathCell::ToTeX(all);
 }
 
@@ -539,6 +742,12 @@ wxString GroupCell::ToXML(bool all)
     case GC_TYPE_SUBSECTION:
       str += wxT(" type=\"subsection\"");
       break;
+    case GC_TYPE_PAGEBREAK:
+      {
+        str += wxT(" type=\"pagebreak\"/>");
+        return str + MathCell::ToXML(all);
+      }
+      break;
     default:
       str += wxT(" type=\"unknown\"");
       break;
@@ -566,13 +775,30 @@ wxString GroupCell::ToXML(bool all)
       }
       break;
     case GC_TYPE_IMAGE:
+      if (input != NULL)
+        str += input->ToXML(false);
       if (output != NULL)
         str += output->ToXML(true);
       break;
     case GC_TYPE_TEXT:
+      if (input)
+        str += input->ToXML(false);
+      break;
     case GC_TYPE_TITLE:
     case GC_TYPE_SECTION:
     case GC_TYPE_SUBSECTION:
+      if (input)
+        str += input->ToXML(false);
+      if (m_hiddenTree) {
+        str += wxT("<fold>");
+        GroupCell *tmp = m_hiddenTree;
+        while (tmp) {
+          str += tmp->ToXML(false);
+          tmp = (GroupCell *)tmp->m_next;
+        }
+        str += wxT("</fold>");
+      }
+      break;
     default:
       if (output != NULL)
         str += output->ToXML(false);
@@ -589,7 +815,7 @@ void GroupCell::SelectRectGroup(wxRect& rect, wxPoint& one, wxPoint& two,
   *first = NULL;
   *last = NULL;
 
-  if (m_input->ContainsRect(rect))
+  if ((m_input) && (m_input->ContainsRect(rect)))
     m_input->SelectRect(rect, first, last);
   else if (m_output != NULL && !m_hide && m_outputRect.Contains(rect))
     SelectRectInOutput(rect, one, two, first, last);
@@ -704,18 +930,35 @@ bool GroupCell::SetEditableContent(wxString text)
 
 MathCell *GroupCell::GetEditable()
 {
-  if (m_groupType == GC_TYPE_IMAGE)
-    return NULL;
-  if (m_groupType != GC_TYPE_CODE)
-    return GetLabel();
-  return GetInput();
+  switch (m_groupType) {
+    case GC_TYPE_CODE:
+      return GetInput();
+    case GC_TYPE_IMAGE:
+      return GetInput();
+    case GC_TYPE_TEXT:
+      return GetInput();
+    case GC_TYPE_TITLE:
+      return GetInput();
+    case GC_TYPE_SECTION:
+      return GetInput();
+    case GC_TYPE_SUBSECTION:
+      return GetInput();
+    case GC_TYPE_PAGEBREAK:
+    default:
+      return NULL;
+  }
 }
 
 void GroupCell::BreakLines(int fullWidth)
+{
+  BreakLines(m_output, fullWidth);
+}
+
+void GroupCell::BreakLines(MathCell *cell, int fullWidth)
 {
   int currentWidth = m_indent;
 
-  MathCell *tmp = m_output;
+  MathCell *tmp = cell;
 
   while (tmp != NULL && !m_hide) {
     tmp->ResetData();
@@ -733,21 +976,22 @@ void GroupCell::BreakLines(int fullWidth)
 
 void GroupCell::SelectOutput(MathCell **start, MathCell **end)
 {
-
   if (m_hide)
     return;
 
   *start = m_output;
 
-  while (*start != NULL && (*start)->GetType() != MC_TYPE_LABEL)
+  while (*start != NULL && (*start)->GetStyle() != TS_LABEL)
     *start = (*start)->m_nextToDraw;
 
+
   if (*start != NULL)
     *start = (*start)->m_nextToDraw;
 
   *end = *start;
 
-  while (*end != NULL && (*end)->GetType() == MC_TYPE_DEFAULT && (*end)->m_nextToDraw != NULL)
+  while (*end != NULL &&
+         (*end)->m_nextToDraw != NULL)
     *end = (*end)->m_nextToDraw;
 
   if (*end == NULL || *start == NULL)
@@ -756,13 +1000,18 @@ void GroupCell::SelectOutput(MathCell **start, MathCell **end)
 
 void GroupCell::BreakUpCells(CellParser parser, int fontsize, int clientWidth)
 {
-  MathCell *tmp = m_output;
+  BreakUpCells(m_output, parser, fontsize, clientWidth);
+}
+
+void GroupCell::BreakUpCells(MathCell *cell, CellParser parser, int fontsize, int clientWidth)
+{
+  MathCell *tmp = cell;
 
   while (tmp != NULL && !m_hide) {
     if (tmp->GetWidth() > clientWidth) {
       if (tmp->BreakUp()) {
-        tmp->RecalculateWidths(parser, fontsize, false);
-        tmp->RecalculateSize(parser, fontsize, false);
+        tmp->RecalculateWidths(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize, false);
+        tmp->RecalculateSize(parser,  tmp->IsMath() ? m_mathFontSize : m_fontSize, false);
       }
     }
     tmp = tmp->m_nextToDraw;
@@ -779,3 +1028,210 @@ void GroupCell::UnBreakUpCells()
     tmp = tmp->m_next;
   }
 }
+
+// support for hiding text, code cells
+
+void GroupCell::Hide(bool hide) {
+  if (IsFoldable())
+    return;
+
+  if (m_hide == hide)
+    return;
+  else
+  {
+    if (m_hide == false) {
+      if ((m_groupType == GC_TYPE_TEXT) || (m_groupType == GC_TYPE_CODE))
+        ((EditorCell *)GetEditable())->SetFirstLineOnly(true);
+      m_hide = true;
+    }
+    else {
+      if ((m_groupType == GC_TYPE_TEXT) || (m_groupType == GC_TYPE_CODE))
+        ((EditorCell *)GetEditable())->SetFirstLineOnly(false);
+      m_hide = false;
+    }
+
+    ResetSize();
+    GetEditable()->ResetSize();
+  }
+}
+
+void GroupCell::SwitchHide() {
+  Hide(!m_hide);
+}
+
+//
+// support for folding/unfolding sections
+//
+bool GroupCell::HideTree(GroupCell *tree)
+{
+  if (m_hiddenTree)
+    return false;
+  m_hiddenTree = tree;
+  return true;
+
+}
+
+GroupCell *GroupCell::UnhideTree()
+{
+  GroupCell *tree = m_hiddenTree;
+  m_hiddenTree = NULL;
+  return tree;
+}
+
+GroupCell *GroupCell::Fold() {
+  if (!IsFoldable() || m_hiddenTree) // already folded?? shouldn't happen
+    return NULL;
+  if (m_next == NULL)
+    return NULL;
+  int nextgct = ((GroupCell *)m_next)->GetGroupType(); // groupType of the next cell
+  if ((m_groupType == nextgct) || IsLesserGCType(nextgct))
+    return NULL; // if the next gc shouldn't be folded, exit
+
+  // now there is at least one cell to fold (at least m_next)
+  GroupCell *end = (GroupCell *)m_next;
+  GroupCell *start = end; // first to fold
+
+  while (end) {
+    GroupCell *tmp = (GroupCell *)end->m_next;
+    if (tmp == NULL)
+      break;
+    if ((m_groupType == tmp->GetGroupType()) || IsLesserGCType(tmp->GetGroupType()))
+      break; // the next one of the end is not suitable for folding, break
+    end = tmp;
+  }
+
+  // cell(s) to fold are between start and end (including these two)
+
+  MathCell *next = end->m_next;
+  m_next = m_nextToDraw = next; // may be NULL, it's ok
+  if (next)
+    next->m_previous = next->m_previousToDraw = this;
+
+  start->m_previous = start->m_previousToDraw = NULL;
+  end->m_next = end->m_nextToDraw = NULL;
+  m_hiddenTree = start; // save the torn out tree into m_hiddenTree
+  return this;
+}
+
+// unfolds the m_hiddenTree beneath this cell
+// be careful to update m_last if this happens in the main tree in MathCtrl
+GroupCell *GroupCell::Unfold() {
+  if (!IsFoldable() || !m_hiddenTree)
+    return NULL;
+
+  MathCell *next = m_next;
+
+  // sew together this cell with m_hiddenTree
+  m_next = m_nextToDraw = m_hiddenTree;
+  m_hiddenTree->m_previous = m_hiddenTree->m_previousToDraw = this;
+
+  MathCell *tmp = m_hiddenTree;
+  while (tmp->m_next)
+    tmp = tmp->m_next;
+  // tmp holds the last element of m_hiddenTree
+  tmp->m_next = tmp->m_nextToDraw = next;
+  if (next)
+    next->m_previous = next->m_previousToDraw = tmp;
+
+  m_hiddenTree = NULL;
+  return (GroupCell *)tmp;
+}
+
+GroupCell *GroupCell::FoldAll(bool all) {
+  GroupCell *result = this;
+  if (IsFoldable() && !m_hiddenTree) {
+    Fold();
+    if (m_hiddenTree)
+      m_hiddenTree->FoldAll(true);
+  }
+  else
+    result = NULL;
+
+  if (all && m_next)
+    return ((GroupCell *)m_next)->FoldAll(true);
+  else
+    return result;
+}
+
+// unfolds recursivly its contents
+// if (all) then also calls it on it's m_next
+GroupCell *GroupCell::UnfoldAll(bool all) {
+  if (all && m_next)
+    ((GroupCell *)m_next)->UnfoldAll(true);
+
+  if (!IsFoldable() || !m_hiddenTree)
+    return NULL;
+
+  m_hiddenTree->UnfoldAll(true);
+
+  return Unfold();
+}
+
+bool GroupCell::IsLesserGCType(int comparedTo) {
+  switch (m_groupType) {
+    case GC_TYPE_CODE:
+    case GC_TYPE_TEXT:
+    case GC_TYPE_PAGEBREAK:
+    case GC_TYPE_IMAGE:
+      if ((comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION) ||
+          (comparedTo == GC_TYPE_SUBSECTION))
+        return true;
+      else
+        return false;
+    case GC_TYPE_SUBSECTION:
+      if ((comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION))
+        return true;
+      else
+        return false;
+    case GC_TYPE_SECTION:
+      if (comparedTo == GC_TYPE_TITLE)
+        return true;
+      else
+        return false;
+    case GC_TYPE_TITLE:
+      return false;
+    default:
+      return false;
+  }
+}
+
+void GroupCell::Number(int &section, int &subsection, int &image) {
+  switch (m_groupType) {
+    case GC_TYPE_TITLE:
+      section = subsection = 0;
+      break;
+    case GC_TYPE_SECTION:
+      section++;
+      subsection = 0;
+      {
+        wxString num = wxT(" ");
+        num << section << wxT(" ");
+        ((TextCell*)m_input)->SetValue(num);
+      }
+      break;
+    case GC_TYPE_SUBSECTION:
+      subsection++;
+      {
+        wxString num = wxT("  ");
+        num << section << wxT(".") << subsection << wxT(" ");
+        ((TextCell*)m_input)->SetValue(num);
+      }
+      break;
+    case GC_TYPE_IMAGE:
+      image++;
+      {
+        wxString num = _("Figure");
+        num << wxT(" ") << image << wxT(":");
+        ((TextCell*)m_input)->SetValue(num);
+      }
+      break;
+    default:
+      break;
+  }
+
+  if (IsFoldable() && m_hiddenTree)
+    m_hiddenTree->Number(section, subsection, image);
+
+  if (m_next)
+    ((GroupCell *)m_next)->Number(section, subsection, image);
+}
