@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2008 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <wx/config.h>
 #include <wx/tokenzr.h>
 #include <wx/sstream.h>
+#include <wx/regex.h>
 
 #include "MathParser.h"
 
@@ -86,7 +87,7 @@ MathCell* MathParser::ParseCellTag(wxXmlNode* node)
     wxXmlNode *children = node->GetChildren();
     while (children) {
       if (children->GetName() == wxT("input")) {
-        MathCell *editor = ParseTag(node->GetChildren());
+        MathCell *editor = ParseTag(children->GetChildren());
         group->SetEditableContent(editor->GetValue());
         delete editor;
       }
@@ -97,7 +98,26 @@ MathCell* MathParser::ParseCellTag(wxXmlNode* node)
   }
   else if (type == wxT("image")) {
     group = new GroupCell(GC_TYPE_IMAGE);
-    group->AppendOutput(ParseTag(node->GetChildren()));
+    wxXmlNode *children = node->GetChildren();
+    while (children) {
+      if (children->GetName() == wxT("editor")) {
+        MathCell *ed = ParseEditorTag(children);
+        group->SetEditableContent(ed->GetValue());
+        delete ed;
+      }
+      else
+        group->AppendOutput(ParseTag(children));
+      children = children->GetNext();
+    }
+  }
+  else if (type == wxT("pagebreak")) {
+    group = new GroupCell(GC_TYPE_PAGEBREAK);
+  }
+  else if (type == wxT("text")) {
+    group = new GroupCell(GC_TYPE_TEXT);
+    MathCell *editor = ParseTag(node->GetChildren());
+    group->SetEditableContent(editor->GetValue());
+    delete editor;
   }
   else {
     // text types
@@ -105,14 +125,39 @@ MathCell* MathParser::ParseCellTag(wxXmlNode* node)
       group = new GroupCell(GC_TYPE_TITLE);
     else if (type == wxT("section"))
       group = new GroupCell(GC_TYPE_SECTION);
-    else if (type == wxT("text"))
-      group = new GroupCell(GC_TYPE_TEXT);
+    else if (type == wxT("subsection"))
+      group = new GroupCell(GC_TYPE_SUBSECTION);
     else
       return NULL;
 
-    MathCell *editor = ParseTag(node->GetChildren());
-    group->SetEditableContent(editor->GetValue());
-    delete editor;
+    wxXmlNode *children = node->GetChildren();
+    while (children) {
+      if (children->GetName() == wxT("editor")) {
+        MathCell *ed = ParseEditorTag(children);
+        group->SetEditableContent(ed->GetValue());
+        delete ed;
+      }
+      else if (children->GetName() == wxT("fold")) { // we have folded groupcells
+        wxXmlNode *xmlcells = children->GetChildren();
+        MathCell *tree = NULL;
+        MathCell *last = NULL;
+        if (xmlcells) {
+          last = tree = ParseTag(xmlcells, false); // first cell
+          while (xmlcells->GetNext()) {
+            xmlcells = xmlcells->GetNext();
+            MathCell *cell = ParseTag(xmlcells, false);
+
+            last->m_next = last->m_nextToDraw = cell;
+            last->m_next->m_previous = last->m_next->m_previousToDraw = last;
+
+            last = last->m_next;
+          }
+          if (tree)
+            group->HideTree((GroupCell *)tree);
+        }
+      }
+      children = children->GetNext();
+    }
   }
 
   group->SetParent(group, false);
@@ -132,6 +177,8 @@ MathCell* MathParser::ParseEditorTag(wxXmlNode* node)
     editor->SetType(MC_TYPE_TITLE);
   else if (type == wxT("section"))
     editor->SetType(MC_TYPE_SECTION);
+  else if (type == wxT("subsection"))
+    editor->SetType(MC_TYPE_SUBSECTION);
 
   wxString text = wxEmptyString;
   wxXmlNode *line = node->GetChildren();
@@ -171,6 +218,7 @@ MathCell* MathParser::ParseFracTag(wxXmlNode* node)
         frac->SetFracStyle(FC_CHOOSE);
       }
       frac->SetType(m_ParserStyle);
+      frac->SetStyle(TS_VARIABLE);
       frac->SetupBreakUps();
       return frac;
     }
@@ -194,6 +242,7 @@ MathCell* MathParser::ParseDiffTag(wxXmlNode* node)
     {
       diff->SetBase(ParseTag(child, true));
       diff->SetType(m_ParserStyle);
+      diff->SetStyle(TS_VARIABLE);
       return diff;
     }
   }
@@ -217,6 +266,7 @@ MathCell* MathParser::ParseSupTag(wxXmlNode* node)
       power->SetExponentFlag();
       expt->SetPower(power);
       expt->SetType(m_ParserStyle);
+      expt->SetStyle(TS_VARIABLE);
       return expt;
     }
   }
@@ -244,6 +294,7 @@ MathCell* MathParser::ParseSubSupTag(wxXmlNode* node)
         power->SetExponentFlag();
         subsup->SetExponent(power);
         subsup->SetType(m_ParserStyle);
+        subsup->SetStyle(TS_VARIABLE);
         return subsup;
       }
     }
@@ -266,6 +317,7 @@ MathCell* MathParser::ParseSubTag(wxXmlNode* node)
       index->SetExponentFlag();
       sub->SetIndex(index);
       sub->SetType(m_ParserStyle);
+      sub->SetStyle(TS_VARIABLE);
       return sub;
     }
   }
@@ -286,6 +338,7 @@ MathCell* MathParser::ParseAtTag(wxXmlNode* node)
     {
       at->SetIndex(ParseTag(child, false));
       at->SetType(m_ParserStyle);
+      at->SetStyle(TS_VARIABLE);
       return at;
     }
   }
@@ -304,6 +357,7 @@ MathCell* MathParser::ParseFunTag(wxXmlNode* node)
     if (child)
     {
       fun->SetType(m_ParserStyle);
+      fun->SetStyle(TS_VARIABLE);
       fun->SetArg(ParseTag(child, false));
       return fun;
     }
@@ -327,6 +381,27 @@ MathCell* MathParser::ParseText(wxXmlNode* node, int style)
       if (str.Length() > 100) // This could be made configurable.
         str = str.Left(30) + wxString::Format(wxT("[%d digits]"), str.Length() - 60) + str.Right(30);
     }
+    cell->SetType(m_ParserStyle);
+    cell->SetStyle(style);
+    cell->SetHighlight(m_highlight);
+    cell->SetValue(str);
+  }
+  return cell;
+}
+
+MathCell* MathParser::ParseCharCode(wxXmlNode* node, int style)
+{
+  TextCell* cell = new TextCell;
+  wxString str;
+  if (node != NULL && (str = node->GetContent()) != wxEmptyString)
+  {
+    long code;
+    if (str.ToLong(&code))
+      str = wxString::Format(wxT("%c"), code);
+#if !wxUSE_UNICODE
+    wxString str1(str.wc_str(wxConvUTF8), *wxConvCurrent);
+    str = str1;
+#endif
     cell->SetValue(str);
     cell->SetType(m_ParserStyle);
     cell->SetStyle(style);
@@ -341,6 +416,7 @@ MathCell* MathParser::ParseSqrtTag(wxXmlNode* node)
   SqrtCell* cell = new SqrtCell;
   cell->SetInner(ParseTag(child, true));
   cell->SetType(m_ParserStyle);
+  cell->SetStyle(TS_VARIABLE);
   cell->SetHighlight(m_highlight);
   return cell;
 }
@@ -351,6 +427,7 @@ MathCell* MathParser::ParseAbsTag(wxXmlNode* node)
   AbsCell* cell = new AbsCell;
   cell->SetInner(ParseTag(child, true));
   cell->SetType(m_ParserStyle);
+  cell->SetStyle(TS_VARIABLE);
   cell->SetHighlight(m_highlight);
   return cell;
 }
@@ -361,6 +438,7 @@ MathCell* MathParser::ParseParenTag(wxXmlNode* node)
   ParenCell* cell = new ParenCell;
   cell->SetInner(ParseTag(child, true), m_ParserStyle);
   cell->SetHighlight(m_highlight);
+  cell->SetStyle(TS_VARIABLE);
   if (node->GetProperties() != NULL)
     cell->SetPrint(false);
   return cell;
@@ -382,6 +460,7 @@ MathCell* MathParser::ParseLimitTag(wxXmlNode* node)
       {
         limit->SetBase(ParseTag(child, false));
         limit->SetType(m_ParserStyle);
+        limit->SetStyle(TS_VARIABLE);
         return limit;
       }
     }
@@ -394,7 +473,8 @@ MathCell* MathParser::ParseSumTag(wxXmlNode* node)
 {
   SumCell *sum = new SumCell;
   wxXmlNode* child = node->GetChildren();
-  if (node->GetProperties() != NULL)
+  wxString type = node->GetPropVal(wxT("type"), wxT("sum"));
+  if (type == wxT("prod"))
     sum->SetSumStyle(SM_PROD);
   sum->SetHighlight(m_highlight);
   if (child)
@@ -403,12 +483,14 @@ MathCell* MathParser::ParseSumTag(wxXmlNode* node)
     child = child->GetNext();
     if (child)
     {
-      sum->SetOver(ParseTag(child, false));
+      if (type != wxT("lsum"))
+        sum->SetOver(ParseTag(child, false));
       child = child->GetNext();
       if (child)
       {
         sum->SetBase(ParseTag(child, false));
         sum->SetType(m_ParserStyle);
+        sum->SetStyle(TS_VARIABLE);
         return sum;
       }
     }
@@ -441,6 +523,7 @@ MathCell* MathParser::ParseIntTag(wxXmlNode* node)
           {
             in->SetVar(ParseTag(child, true));
             in->SetType(m_ParserStyle);
+            in->SetStyle(TS_VARIABLE);
             return in;
           }
         }
@@ -457,6 +540,7 @@ MathCell* MathParser::ParseIntTag(wxXmlNode* node)
       {
         in->SetVar(ParseTag(child, true));
         in->SetType(m_ParserStyle);
+        in->SetStyle(TS_VARIABLE);
         return in;
       }
     }
@@ -487,6 +571,7 @@ MathCell* MathParser::ParseTableTag(wxXmlNode* node)
     rows = rows->GetNext();
   }
   matrix->SetType(m_ParserStyle);
+  matrix->SetStyle(TS_VARIABLE);
   matrix->SetDimension();
   return matrix;
 }
@@ -496,6 +581,8 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
   //  wxYield();
   MathCell* tmp = NULL;
   MathCell* cell = NULL;
+  bool warning = all;
+
   while (node)
   {
     // Parse tags
@@ -669,7 +756,7 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
         else
           cell->AppendCell(ParseTableTag(node));
       }
-      else if (tagName == wxT("mth"))
+      else if ((tagName == wxT("mth")) || (tagName == wxT("line")))
       {
         MathCell *tmp = ParseTag(node->GetChildren());
         if (tmp != NULL)
@@ -683,7 +770,6 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
       }
       else if (tagName == wxT("lbl"))
       {
-        int oldPS = m_ParserStyle;
         MathCell* tmp = ParseText(node->GetChildren(), TS_LABEL);
         tmp->ForceBreakLine(true);
         if (cell == NULL)
@@ -713,6 +799,10 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
       else if (tagName == wxT("img"))
       {
         wxString filename(node->GetChildren()->GetContent());
+#if !wxUSE_UNICODE
+        wxString filename1(filename.wc_str(wxConvUTF8), *wxConvCurrent);
+        filename = filename1;
+#endif
 
         ImgCell *tmp;
 
@@ -723,6 +813,9 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
         else
           tmp = new ImgCell(filename, false, NULL);
 
+        if (node->GetPropVal(wxT("rect"), wxT("true")) == wxT("false"))
+          tmp->DrawRectangle(false);
+
         if (cell == NULL)
           cell = tmp;
         else
@@ -737,7 +830,13 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
         while (tokens.HasMoreTokens()) {
           wxString token = tokens.GetNextToken();
           if (token.Length())
+          {
+#if !wxUSE_UNICODE
+            wxString token1(token.wc_str(wxConvUTF8), *wxConvCurrent);
+            token = token1;
+#endif
             images.Add(token);
+          }
         }
         tmp->LoadImages(images);
         if (cell == NULL)
@@ -759,6 +858,13 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
         else
           cell->AppendCell(ParseCellTag(node));
       }
+      else if (tagName == wxT("ascii"))
+      {
+        if (cell == NULL)
+          cell = ParseCharCode(node->GetChildren());
+        else
+          cell->AppendCell(ParseCharCode(node->GetChildren()));
+      }
       else if (node->GetChildren())
       {
         if (cell == NULL)
@@ -778,10 +884,19 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
     if (!all)
       break;
 
-    if (tmp == NULL)
-      tmp = cell;
-    else
-      cell = cell->m_next;
+    if (cell != NULL)
+    {
+      if (tmp == NULL)
+        tmp = cell;
+      else
+        cell = cell->m_next;
+    }
+    else if (warning)
+    {
+      wxMessageBox(_("Parts of the document will not be loaded correctly!"), _("Warning"),
+        wxOK | wxICON_WARNING);
+      warning = false;
+    }
 
     node = node->GetNext();
   }
@@ -806,6 +921,14 @@ MathCell* MathParser::ParseLine(wxString s, int style)
   bool showLong = false;
   config->Read(wxT("showLong"), &showLong);
 
+  wxRegEx graph(wxT("[[:cntrl:]]"));
+
+#if wxUSE_UNICODE
+  graph.Replace(&s, wxT("\xFFFD"));
+#else
+  graph.Replace(&s, wxT("?"));
+#endif
+
   if (s.Length() < MAXLENGTH || showLong)
   {
 
