@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ bool MathPrintout::OnPrintPage(int num)
 {
   double screenScaleX, screenScaleY;
   double ppiScale;
-  MathCell* tmp = m_tree;
+  GroupCell* tmp;
   wxDC* dc = GetDC();
 
   int marginX, marginY;
@@ -71,11 +71,16 @@ bool MathPrintout::OnPrintPage(int num)
   dc->SetUserScale(screenScaleX, screenScaleY);
 
   // Go to current page
-  tmp = m_pages[num - 1];
+  tmp = (GroupCell *)m_pages[num - 1];
 
   // Print page
   if (tmp != NULL)
   {
+    if (tmp->GetGroupType() == GC_TYPE_PAGEBREAK)
+      tmp = (GroupCell *)tmp->m_next;
+    if (tmp == NULL)
+      return true;
+
     wxPoint point;
     point.x = marginX;
     point.y = marginY + tmp->GetMaxCenter() + GetHeaderHeight();
@@ -90,7 +95,7 @@ bool MathPrintout::OnPrintPage(int num)
 
     parser.SetIndent(marginX);
 
-    while (tmp != NULL)
+    while (tmp != NULL && tmp->GetGroupType() != GC_TYPE_PAGEBREAK)
     {
       tmp->Draw(parser, point, fontsize, false);
       if (tmp->m_next != NULL) {
@@ -100,7 +105,7 @@ bool MathPrintout::OnPrintPage(int num)
         drop = tmp->m_next->GetMaxDrop();
       }
 
-      tmp = tmp->m_next;
+      tmp = (GroupCell *)tmp->m_next;
       if (tmp == NULL || tmp->BreakPageHere())
         break;
     }
@@ -132,7 +137,7 @@ void MathPrintout::BreakPages()
   int currentHeight = marginY;
   int skip = SCALE_PX(MC_GROUP_SKIP, scale);;
 
-  MathCell* tmp = m_tree;
+  GroupCell* tmp = (GroupCell *)m_tree;
   m_pages.push_back(tmp);
 
   m_numberOfPages = 1;
@@ -140,9 +145,11 @@ void MathPrintout::BreakPages()
   {
     tmp->BreakPage(false);
 
-    if (currentHeight + tmp->GetMaxHeight() + skip >= pageHeight - marginY)
+    if (currentHeight + tmp->GetMaxHeight() + skip >= pageHeight - marginY ||
+        tmp->GetGroupType() == GC_TYPE_PAGEBREAK)
     {
-      currentHeight = marginY + tmp->GetMaxHeight() + headerHeight;
+      if (tmp->GetGroupType() != GC_TYPE_PAGEBREAK)
+        currentHeight = marginY + tmp->GetMaxHeight() + headerHeight;
       tmp->BreakPage(true);
       m_pages.push_back(tmp);
       m_numberOfPages++;
@@ -150,14 +157,13 @@ void MathPrintout::BreakPages()
     else
       currentHeight += tmp->GetMaxHeight() + skip;
 
-    tmp = tmp->m_next;
+    tmp = (GroupCell *)tmp->m_next;
   }
 }
 
 void MathPrintout::SetupData()
 {
-  RecalculateWidths();
-  RecalculateSize();
+  Recalculate();
   BreakPages();
 }
 
@@ -230,13 +236,13 @@ void MathPrintout::PrintHeader(int pageNum, wxDC* dc, double scale)
   dc->SetTextForeground(wxColour(wxT("black")));
   dc->SetPen(wxPen(wxT("black"), 1, wxSOLID));
 }
-
+/*
 void MathPrintout::RecalculateSize()
 {
   wxConfig *config = (wxConfig *)wxConfig::Get();
   int fontsize = 12;
   config->Read(wxT("fontSize"), &fontsize);
-  MathCell* tmp = m_tree;
+  GroupCell* tmp = m_tree;
   double scale = GetPPIScale();
 
   wxDC *dc = GetDC();
@@ -252,13 +258,15 @@ void MathPrintout::RecalculateSize()
     tmp = tmp->m_next;
   }
 }
-
-void MathPrintout::RecalculateWidths()
+*/
+void MathPrintout::Recalculate()
 {
   wxConfig *config = (wxConfig *)wxConfig::Get();
   int fontsize = 12;
   config->Read(wxT("fontSize"), &fontsize);
-  MathCell* tmp = m_tree;
+  int mfontsize = fontsize;
+  config->Read(wxT("mathfontsize"), &mfontsize);
+  GroupCell* tmp = (GroupCell *)m_tree;
   double scale = GetPPIScale();
 
   wxDC *dc = GetDC();
@@ -277,8 +285,8 @@ void MathPrintout::RecalculateWidths()
 
   while (tmp != NULL)
   {
-    tmp->RecalculateWidths(parser, fontsize, false);
-    tmp = tmp->m_next;
+    tmp->Recalculate(parser, fontsize, mfontsize);
+    tmp = (GroupCell *)tmp->m_next;
   }
 }
 
