@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -211,7 +211,7 @@ wxString MatrCell::ToString(bool all)
 
 wxString MatrCell::ToTeX(bool all)
 {
-  wxString s = wxT("\\pmatrix{");
+  wxString s = wxT("\\begin{pmatrix}");
   for (int i = 0; i < m_matHeight; i++)
   {
     for (int j = 0; j < m_matWidth; j++)
@@ -223,7 +223,7 @@ wxString MatrCell::ToTeX(bool all)
     if (i < m_matHeight - 1)
       s += wxT("\\cr ");
   }
-  s += wxT("}");
+  s += wxT("\\end{pmatrix}");
   s += MathCell::ToTeX(all);
   return s;
 }
