@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -41,20 +41,27 @@ public:
   wxString GetDiffPart();
   bool IsOperator();
   wxString GetValue() { return m_text; }
-  void SetStyle(int style) { m_textStyle = style; }
-  int GetStyle() { return m_textStyle; }
-  wxString GetSymbolString(CellParser& parser);
-  wxString GetGreekString(CellParser& parser);
-#if wxUSE_UNICODE || defined (__WXGTK20__) || defined (__WXMAC__)
-  wchar_t* GetGreekStringUnicode();
-#else
-  wxString GetGreekStringIso();
+  wxString GetGreekStringTeX();
+  wxString GetSymbolTeX();
+#if wxUSE_UNICODE
+  wxString GetGreekStringUnicode();
+  wxString GetSymbolUnicode();
+#elif defined __WXMSW__
+  wxString GetGreekStringSymbol();
+  wxString GetSymbolSymbol();
 #endif
   bool IsShortNum();
 protected:
+  void SetAltText();
   wxString m_text;
+  wxString m_altText, m_altJsText;
+  wxString m_fontname, m_texFontname;
+  bool m_alt, m_altJs;
   int m_realCenter;
   int m_fontSize;
+  int m_fontSizeTeX;
+  int m_fontSizeLabel;
+  int m_labelWidth, m_labelHeight;
 };
 
 #endif //_TEXTCELL_H_
