@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 #include <wx/socket.h>
 #include <wx/config.h>
 #include <wx/process.h>
+#include <wx/fdrepdlg.h>
+#include <wx/regex.h>
 
 #if defined (__WXMSW__)
  #include <wx/msw/helpchm.h>
@@ -35,7 +37,7 @@
 
 #define SOCKET_SIZE 1024
 #define DOCUMENT_VERSION_MAJOR 1
-#define DOCUMENT_VERSION_MINOR 0
+#define DOCUMENT_VERSION_MINOR 1
 
 class MyApp : public wxApp
 {
@@ -67,7 +69,7 @@ public:
   {
     m_openFile = file;
   }
-  void SendMaxima(wxString s);
+  void SendMaxima(wxString s, bool history = false);
   void OpenFile(wxString file,
                 wxString command = wxEmptyString); // Open a file
 protected:
@@ -88,8 +90,15 @@ protected:
   void UpdateMenus(wxUpdateUIEvent& event);        //
   void UpdateToolBar(wxUpdateUIEvent& event);      //
   void UpdateSlider(wxUpdateUIEvent& event);       //
+  void ShowPane(wxCommandEvent& event);            //
   void OnProcessEvent(wxProcessEvent& event);      //
   void PopupMenu(wxCommandEvent& event);           //
+  void StatsMenu(wxCommandEvent& event);           //
+
+  void OnFind(wxFindDialogEvent& event);
+  void OnFindClose(wxFindDialogEvent& event);
+  void OnReplace(wxFindDialogEvent& event);
+  void OnReplaceAll(wxFindDialogEvent& event);
 
   void ServerEvent(wxSocketEvent& event);          // server event: maxima connection
   void ClientEvent(wxSocketEvent& event);          // client event: maxima input/output
@@ -97,16 +106,17 @@ protected:
   void ConsoleAppend(wxString s, int type);        // append maxima output to console
   void DoConsoleAppend(wxString s, int type,       //
                        bool newLine = true, bool bigSkip = true);
-  void DoRawConsoleAppend(wxString s, int type,    //
-                          bool newLine = true,     //
-                          bool hide = false);      //
+  void DoRawConsoleAppend(wxString s, int type);   //
 
   void EditInputMenu(wxCommandEvent& event);       //
-  void EvaluateEvent(wxCommandEvent& event);     //
+  void EvaluateEvent(wxCommandEvent& event);       //
   void InsertMenu(wxCommandEvent& event);          //
   void SliderEvent(wxScrollEvent& event);
+  void HistoryDClick(wxCommandEvent& event);
+  void OnInspectorEvent(wxCommandEvent& ev);
   void DumpProcessOutput();
   void TryEvaluateNextInQueue();
+  void TryUpdateInspector();
 
 #if WXM_PRINT
   void CheckForPrintingSupport();
@@ -126,6 +136,7 @@ protected:
   void ReadPrompt();                 // reads prompts
   void ReadMath();                   // reads output other than prompts
   void ReadLispError();              // lisp errors (no prompt prefix/suffix)
+  void ReadLoadSymbols();            // functions after load command
 #ifndef __WXMSW__
   void ReadProcessOutput();          // reads output of maxima command
 #endif
@@ -170,11 +181,17 @@ protected:
   bool m_fileSaved;
   bool m_variablesOK;
   wxString m_helpFile;
+  wxString m_maximaVersion;
+  wxString m_lispVersion;
 #if defined (__WXMSW__)
   wxCHMHelpController m_helpCtrl;
 #else
   wxHtmlHelpController m_helpCtrl;
 #endif
+  wxFindReplaceDialog *m_findDialog;
+  wxFindReplaceData m_findData;
+  wxRegEx m_funRegEx;
+  wxRegEx m_varRegEx;
   DECLARE_EVENT_TABLE()
 };
 
