@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -30,52 +30,13 @@ wxMaximaFrame::wxMaximaFrame(wxWindow* parent, int id, const wxString& title,
                              long style):
     wxFrame(parent, id, title, pos, size, wxDEFAULT_FRAME_STYLE)
 {
-  panel = new wxPanel(this, -1);
-
-  // buttons
-  int panelSize = 1;
-  wxConfig::Get()->Read(wxT("panelSize"), &panelSize);
-  if (panelSize == 2)
-  {
-    button_2 = new wxButton(panel, button_ratsimp, _("Simplify"));
-    button_3 = new wxButton(panel, button_radcan, _("Simplify (r)"));
-    button_4 = new wxButton(panel, button_factor, _("Factor"));
-    button_5 = new wxButton(panel, button_expand, _("Expand"));
-    button_6 = new wxButton(panel, button_trigsimp, _("Simplify (tr)"));
-    button_7 = new wxButton(panel, button_trigexpand, _("Expand (tr)"));
-    button_8 = new wxButton(panel, button_trigreduce, _("Reduce (tr)"));
-    button_9 = new wxButton(panel, button_rectform, _("Rectform"));
-    button_10 = new wxButton(panel, button_sum, _("Sum..."));
-    button_11 = new wxButton(panel, button_product, _("Product..."));
-    button_12 = new wxButton(panel, button_solve, _("Solve..."));
-    button_13 = new wxButton(panel, button_solve_ode, _("Solve ODE..."));
-    button_14 = new wxButton(panel, button_diff, _("Diff..."));
-    button_15 = new wxButton(panel, button_integrate, _("Integrate..."));
-    button_16 = new wxButton(panel, button_limit, _("Limit..."));
-    button_17 = new wxButton(panel, button_taylor, _("Series..."));
-    button_18 = new wxButton(panel, button_subst, _("Subst..."));
-    button_19 = new wxButton(panel, button_map, _("Map..."));
-    button_20 = new wxButton(panel, button_plot2, _("Plot 2D..."));
-    button_21 = new wxButton(panel, button_plot3, _("Plot 3D..."));
-  }
-  else if (panelSize == 1)
-  {
-    button_2 = new wxButton(panel, button_ratsimp, _("Simplify"));
-    button_3 = new wxButton(panel, button_radcan, _("Simplify (r)"));
-    button_4 = new wxButton(panel, button_factor, _("Factor"));
-    button_5 = new wxButton(panel, button_expand, _("Expand"));
-    button_12 = new wxButton(panel, button_solve, _("Solve..."));
-    button_20 = new wxButton(panel, button_plot2, _("Plot 2D..."));
-    button_6 = new wxButton(panel, button_trigsimp, _("Simplify (tr)"));
-    button_7 = new wxButton(panel, button_trigexpand, _("Expand (tr)"));
-    button_8 = new wxButton(panel, button_trigreduce, _("Reduce (tr)"));
-    button_9 = new wxButton(panel, button_rectform, _("Rectform"));
-    button_13 = new wxButton(panel, button_solve_ode, _("Solve ODE..."));
-    button_21 = new wxButton(panel, button_plot3, _("Plot 3D..."));
-  }
+  m_manager.SetManagedWindow(this);
 
   // console
-  m_console = new MathCtrl(panel, -1, wxDefaultPosition, wxDefaultSize);
+  m_console = new MathCtrl(this, -1, wxDefaultPosition, wxDefaultSize);
+
+  // history
+  m_history = new History(this, -1);
 
   SetupMenu();
 #if defined (__WXMSW__) || defined (__WXGTK20__) || defined (__WXMAC__)
@@ -98,6 +59,18 @@ wxMaximaFrame::wxMaximaFrame(wxWindow* parent, int id, const wxString& title,
 
   set_properties();
   do_layout();
+
+  m_console->SetFocus();
+}
+
+wxMaximaFrame::~wxMaximaFrame()
+{
+  wxString perspective = m_manager.SavePerspective();
+
+  wxConfig::Get()->Write(wxT("AUI/perspective"), perspective);
+  wxConfig::Get()->Write(wxT("AUI/toolbar"), (GetToolBar() != NULL));
+
+  m_manager.UnInit();
 }
 
 void wxMaximaFrame::set_properties()
@@ -118,80 +91,66 @@ void wxMaximaFrame::set_properties()
 
 void wxMaximaFrame::do_layout()
 {
-  int panelSize = 1;
-  wxConfig::Get()->Read(wxT("panelSize"), &panelSize);
+  m_manager.AddPane(m_console,
+      wxAuiPaneInfo().Name(wxT("console")).
+                      Center().
+                      CloseButton(false).
+                      CaptionVisible(false).
+                      PaneBorder(false));
 
-  wxFlexGridSizer* grid_sizer_1;
-  if (panelSize > 1)
-    grid_sizer_1 = new wxFlexGridSizer(3, 1, 0, 0);
-  else
-    grid_sizer_1 = new wxFlexGridSizer(2, 1, 0, 0);
+  m_manager.AddPane(m_history,
+      wxAuiPaneInfo().Name(wxT("history")).
+                      Caption(_("History")).
+                      Show(false).
+                      TopDockable(false).
+                      BottomDockable(false).
+                      PaneBorder(true).
+                      Right());
 
-  wxBoxSizer *sizer_1 = new wxBoxSizer(wxHORIZONTAL);
+  m_manager.AddPane(CreateStatPane(),
+      wxAuiPaneInfo().Name(wxT("stats")).
+                      Caption(_("Statistics")).
+                      Show(false).
+                      TopDockable(false).
+                      BottomDockable(false).
+                      PaneBorder(true).
+                      Fixed().
+                      Left());
 
-  wxGridSizer* grid_sizer_2;
-  if (panelSize == 2)
-    grid_sizer_2 = new wxGridSizer(2, 10, 0, 0);
-  else
-    grid_sizer_2 = new wxGridSizer(2, 6, 0, 0);
+  m_manager.AddPane(CreateMathPane(),
+      wxAuiPaneInfo().Name(wxT("math")).
+                      Caption(_("General Math")).
+                      Show(false).
+                      TopDockable(false).
+                      BottomDockable(false).
+                      PaneBorder(true).
+                      Fixed().
+                      Left());
 
-  // buttons
-  if (panelSize == 2)
-  {
-    grid_sizer_2->Add(button_2, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_3, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_4, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_5, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_6, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_7, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_8, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_9, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_10, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_11, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_12, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_13, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_14, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_15, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_16, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_17, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_18, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_19, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_20, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_21, 0, wxALL | wxEXPAND, 0);
+  m_manager.AddPane(CreateFormatPane(),
+      wxAuiPaneInfo().Name(wxT("format")).
+                      Caption(_("Insert")).
+                      Show(false).
+                      TopDockable(false).
+                      BottomDockable(false).
+                      PaneBorder(true).
+                      Fixed().
+                      Left());
+
+  wxConfigBase *config = wxConfig::Get();
+  bool loadPanes = false;
+  config->Read(wxT("AUI/savePanes"), &loadPanes);
+
+  if (loadPanes) {
+    wxString perspective;
+    bool toolbar = true;
+    if (config->Read(wxT("AUI/perspective"), &perspective))
+      m_manager.LoadPerspective(perspective);
+    config->Read(wxT("AUI/toolbar"), &toolbar);
+    ShowToolBar(toolbar);
   }
-  else if (panelSize == 1)
-  {
-    grid_sizer_2->Add(button_2, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_3, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_4, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_5, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_12, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_20, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_6, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_7, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_8, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_9, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_13, 0, wxALL | wxEXPAND, 0);
-    grid_sizer_2->Add(button_21, 0, wxALL | wxEXPAND, 0);
-  }
-
-  // all
-  grid_sizer_1->Add(m_console, 1, wxALL | wxEXPAND, 0);
-
-  if (panelSize > 0)
-    grid_sizer_1->Add(grid_sizer_2, 1, wxALL, 2);
-
-  panel->SetAutoLayout(true);
-  panel->SetSizer(grid_sizer_1);
-  grid_sizer_1->Fit(panel);
-  grid_sizer_1->SetSizeHints(panel);
-  grid_sizer_1->AddGrowableRow(0);
-  grid_sizer_1->AddGrowableCol(0);
-
-  sizer_1->Add(panel, 1, wxEXPAND, 0);
-  SetAutoLayout(true);
-  SetSizer(sizer_1);
-  Layout();
+  else
+    m_manager.Update();
 }
 
 void wxMaximaFrame::SetupMenu()
@@ -222,13 +181,13 @@ void wxMaximaFrame::SetupMenu()
   APPEND_MENU_ITEM(wxglade_tmp_menu_1, menu_save_id, _("&Save\tCtrl-S"),
                    _("Save document"), wxT("gtk-save"));
   APPEND_MENU_ITEM(wxglade_tmp_menu_1, menu_save_as_id, _("Save As...\tShift-Ctrl-S"),
-                   _("Save document as ..."), wxT("gtk-save"));
+                   _("Save document as"), wxT("gtk-save"));
   wxglade_tmp_menu_1->Append(menu_load_id, _("&Load Package...\tCtrl-L"),
                              _("Load a Maxima package file"), wxITEM_NORMAL);
   wxglade_tmp_menu_1->Append(menu_batch_id, _("&Batch File...\tCtrl-B"),
                              _("Load a Maxima file using the batch command"), wxITEM_NORMAL);
-  APPEND_MENU_ITEM(wxglade_tmp_menu_1, menu_export_html, _("&Export..."),
-                   _("Export document to a HTML or LaTeX file"), wxT("stock_export"));
+  wxglade_tmp_menu_1->Append(menu_export_html, _("&Export..."),
+                   _("Export document to a HTML or pdfLaTeX file"), wxITEM_NORMAL);
 #if WXM_PRINT
   wxglade_tmp_menu_1->AppendSeparator();
   APPEND_MENU_ITEM(wxglade_tmp_menu_1, wxID_PRINT, _("&Print...\tCtrl-P"),
@@ -251,7 +210,10 @@ void wxMaximaFrame::SetupMenu()
                              wxITEM_NORMAL);
   APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_copy_from_console, _("&Copy\tCtrl-C"),
                    _("Copy selection"), wxT("gtk-copy"));
-  wxglade_tmp_menu_2->Append(menu_copy_tex_from_console, _("Copy LaTeX"),
+  wxglade_tmp_menu_2->Append(menu_copy_text_from_console, _("Copy as Text\tCtrl-Shift-C"),
+                             _("Copy selection from document as text"),
+                             wxITEM_NORMAL);
+  wxglade_tmp_menu_2->Append(menu_copy_tex_from_console, _("Copy as LaTeX"),
                                   _("Copy selection from document in LaTeX format"),
                                   wxITEM_NORMAL);
 #if defined __WXMSW__ || defined __WXMAC__
@@ -262,6 +224,9 @@ void wxMaximaFrame::SetupMenu()
   wxglade_tmp_menu_2->Append(menu_paste, _("Paste\tCtrl-V"),
                              _("Paste text from clipboard"),
                              wxITEM_NORMAL);
+
+  wxglade_tmp_menu_2->AppendSeparator();
+  wxglade_tmp_menu_2->Append(menu_edit_find, _("Find\tCtrl-F"), _("Find and replace"), wxITEM_NORMAL);
   wxglade_tmp_menu_2->AppendSeparator();
   wxglade_tmp_menu_2->Append(menu_select_all, _("Select All\tCtrl-A"),
                              _("Select all"),
@@ -269,45 +234,22 @@ void wxMaximaFrame::SetupMenu()
   wxglade_tmp_menu_2->Append(menu_copy_to_file, _("Save Selection to Image..."),
                              _("Save selection from document to an image file"),
                              wxITEM_NORMAL);
-  wxMenu* wxglade_tmp_menu_2_sub1 = new wxMenu;
-  wxglade_tmp_menu_2_sub1->Append(menu_cut_input_from_console, _("Cut Cell(s)\tCtrl-Shift-X"),
-                                  _("Cut selected cell(s)"),
-                                  wxITEM_NORMAL);
-  wxglade_tmp_menu_2_sub1->Append(menu_copy_input_from_console, _("Copy Cell(s)\tCtrl-Shift-C"),
-                                  _("Copy selected cell(s)"),
-                                  wxITEM_NORMAL);
-  wxglade_tmp_menu_2_sub1->Append(menu_paste_input, _("Paste Cell(s)\tCtrl-Shift-V"),
-                             _("Paste cell(s) to document"),
-                             wxITEM_NORMAL);
-  APPEND_MENU_ITEM(wxglade_tmp_menu_2_sub1, menu_delete_selection,
-                   _("&Delete Cell(s)"),
-                   _("Delete selected cell(s)"), wxT("gtk-delete"));
-  wxglade_tmp_menu_2_sub1->AppendSeparator();
-  wxglade_tmp_menu_2_sub1->Append(menu_evaluate, _("Evaluate Cell(s)"),
-                             _("Evaluate active or selected cell(s)"), wxITEM_NORMAL);
-  wxglade_tmp_menu_2_sub1->Append(menu_evaluate_all, _("Evaluate All Cells\tCtrl-R"),
-                               _("Evaluate all cells in the document"), wxITEM_NORMAL);
-  wxglade_tmp_menu_2_sub1->AppendSeparator();
-  wxglade_tmp_menu_2_sub1->Append(menu_insert_input, _("New Input &Cell\tF7"),
-                             _("Insert a new input cell"));
-  wxglade_tmp_menu_2_sub1->Append(menu_add_comment, _("New &Text Cell\tF6"),
-                             _("Insert a new text cell"));
-  wxglade_tmp_menu_2_sub1->Append(menu_add_section, _("New &Section Cell\tCtrl-F6"),
-                             _("Insert a new section cell"));
-  wxglade_tmp_menu_2_sub1->Append(menu_add_title, _("New T&itle Cell\tCtrl-Shift-F6"),
-                             _("Insert a new title cell"));
-  wxglade_tmp_menu_2->AppendSeparator();
-  wxglade_tmp_menu_2->Append(wxNewId(), _("Cell"), wxglade_tmp_menu_2_sub1, _("Input"));
-  wxglade_tmp_menu_2->Append(menu_remove_output, _("Remove All Outputs"),
-                            _("Remove output from input cells"), wxITEM_NORMAL);
-  wxglade_tmp_menu_2->Append(menu_insert_image, _("Insert Image..."),
-                              _("Insert image"), wxITEM_NORMAL);
 
   wxglade_tmp_menu_2->AppendSeparator();
-  APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_inc_fontsize, _("Zoom &In\tAlt-I"),
-                   _("Increase fontsize in document"), wxT("gtk-zoom-in"));
-  APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_dec_fontsize, _("Zoom Ou&t\tAlt-O"),
-                   _("Decrease fontsize in document"), wxT("gtk-zoom-out"));
+  APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_zoom_in, _("Zoom &In\tAlt-I"),
+                   _("Zoom in 10%"), wxT("gtk-zoom-in"));
+  APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_zoom_out, _("Zoom Ou&t\tAlt-O"),
+                   _("Zoom out 10%"), wxT("gtk-zoom-out"));
+  // zoom submenu
+  wxMenu* edit_zoom_sub = new wxMenu;
+  edit_zoom_sub->Append(menu_zoom_80,  wxT("80%"),  _("Set zoom to 80%"),  wxITEM_NORMAL);
+  edit_zoom_sub->Append(menu_zoom_100, wxT("100%"), _("Set zoom to 100%"), wxITEM_NORMAL);
+  edit_zoom_sub->Append(menu_zoom_120, wxT("120%"), _("Set zoom to 120%"), wxITEM_NORMAL);
+  edit_zoom_sub->Append(menu_zoom_150, wxT("150%"), _("Set zoom to 150%"), wxITEM_NORMAL);
+  edit_zoom_sub->Append(menu_zoom_200, wxT("200%"), _("Set zoom to 200%"), wxITEM_NORMAL);
+  edit_zoom_sub->Append(menu_zoom_300, wxT("300%"), _("Set zoom to 300%"), wxITEM_NORMAL);
+
+  wxglade_tmp_menu_2->Append(wxNewId(), _("Set Zoom"), edit_zoom_sub, _("Set Zoom"));
   wxglade_tmp_menu_2->Append(menu_fullscreen, _("Full Screen\tAlt-Enter"),
                              _("Toggle full screen editing"),
                              wxITEM_NORMAL);
@@ -316,8 +258,56 @@ void wxMaximaFrame::SetupMenu()
                    _("Configure wxMaxima"), wxT("gtk-preferences"));
   frame_1_menubar->Append(wxglade_tmp_menu_2, _("&Edit"));
 
+  // Cell menu
+  wxglade_tmp_menu_2 = new wxMenu;
+  wxglade_tmp_menu_2->Append(menu_evaluate, _("Evaluate Cell(s)"),
+                             _("Evaluate active or selected cell(s)"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2->Append(menu_evaluate_all, _("Evaluate All Cells\tCtrl-R"),
+                               _("Evaluate all cells in the document"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2->Append(menu_remove_output, _("Remove All Output"),
+                            _("Remove output from input cells"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2->AppendSeparator();
+  wxglade_tmp_menu_2->Append(menu_insert_previous_input, _("Copy Previous Input\tCtrl-I"),
+                                  _("Create a new cell with previous input"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2->Append(menu_autocomplete, _("Complete Word\tCtrl-K"),
+                             _("Complete word"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2->Append(menu_autocomplete_templates, _("Show Template\tCtrl-Shift-K"),
+                             _("Show function template"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2->AppendSeparator();
+  wxglade_tmp_menu_2->Append(menu_insert_input, _("Insert Input &Cell\tF5"),
+                             _("Insert a new input cell"));
+  wxglade_tmp_menu_2->Append(menu_add_comment, _("Insert &Text Cell\tF6"),
+                             _("Insert a new text cell"));
+  wxglade_tmp_menu_2->Append(menu_add_subsection, _("Insert S&ubsection Cell\tF7"),
+                             _("Insert a new subsection cell"));
+  wxglade_tmp_menu_2->Append(menu_add_section, _("Insert &Section Cell\tF8"),
+                             _("Insert a new section cell"));
+  wxglade_tmp_menu_2->Append(menu_add_title, _("Insert T&itle Cell\tF9"),
+                             _("Insert a new title cell"));
+  wxglade_tmp_menu_2->Append(menu_add_pagebreak, _("Insert Page Break\tF10"),
+                             _("Insert a page break"));
+  wxglade_tmp_menu_2->Append(menu_insert_image, _("Insert Image..."),
+                              _("Insert image"), wxITEM_NORMAL);
+  frame_1_menubar->Append(wxglade_tmp_menu_2, _("&Cell"));
+
   // Maxima menu
   wxglade_tmp_menu_2 = new wxMenu;
+
+  // panes
+  wxMenu *wxglade_tmp_menu_2_sub2 = new wxMenu;
+  wxglade_tmp_menu_2_sub2->Append(menu_pane_hideall, _("Hide All\tAlt-Shift--"), _("Hide all panes"), wxITEM_NORMAL);
+  wxglade_tmp_menu_2_sub2->AppendSeparator();
+  wxglade_tmp_menu_2_sub2->AppendCheckItem(menu_pane_math, _("General Math\tAlt-Shift-M"));
+  wxglade_tmp_menu_2_sub2->AppendCheckItem(menu_pane_stats, _("Statistics\tAlt-Shift-S"));
+  wxglade_tmp_menu_2_sub2->AppendCheckItem(menu_pane_history, _("History\tAlt-Shift-H"));
+  wxglade_tmp_menu_2_sub2->AppendCheckItem(menu_pane_format, _("Insert Cell\tAlt-Shift-C"));
+  wxglade_tmp_menu_2_sub2->AppendSeparator();
+  wxglade_tmp_menu_2_sub2->AppendCheckItem(menu_show_toolbar, _("Toolbar\tAlt-Shift-T"));
+  wxglade_tmp_menu_2->Append(wxNewId(), _("Panes"), wxglade_tmp_menu_2_sub2);
+
+  wxglade_tmp_menu_2->AppendSeparator();
+
+
 #if defined (__WXMAC__)
   APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_interrupt_id,
                    _("&Interrupt\tCtrl-."), // command-. interrupts (mac standard)
@@ -333,6 +323,7 @@ void wxMaximaFrame::SetupMenu()
                              _("Delete all values from memory"), wxITEM_NORMAL);
   APPEND_MENU_ITEM(wxglade_tmp_menu_2, menu_add_path, _("Add to &Path..."),
                    _("Add a directory to search path"), wxT("gtk-add"));
+
   wxglade_tmp_menu_2->AppendSeparator();
   wxglade_tmp_menu_2->Append(menu_functions, _("Show &Functions"),
                              _("Show defined functions"), wxITEM_NORMAL);
@@ -345,6 +336,7 @@ void wxMaximaFrame::SetupMenu()
                              _("Delete a function"), wxITEM_NORMAL);
   wxglade_tmp_menu_2->Append(menu_clear_var, _("Delete V&ariable..."),
                              _("Delete a variable"), wxITEM_NORMAL);
+
   wxglade_tmp_menu_2->AppendSeparator();
   wxglade_tmp_menu_2->Append(menu_time, _("Toggle &Time Display"),
                              _("Display time used for evaluation"),
@@ -360,11 +352,16 @@ void wxMaximaFrame::SetupMenu()
   wxMenu* wxglade_tmp_menu_3 = new wxMenu;
   wxglade_tmp_menu_3->Append(menu_solve, _("&Solve..."),
                              _("Solve equation(s)"), wxITEM_NORMAL);
+  wxglade_tmp_menu_3->Append(menu_solve_to_poly, _("Solve (to_poly)..."),
+                             _("Solve equation(s) with to_poly_solver"), wxITEM_NORMAL);
   wxglade_tmp_menu_3->Append(menu_solve_num, _("&Find Root..."),
                              _("Find a root of an equation on an interval"), wxITEM_NORMAL);
   wxglade_tmp_menu_3->Append(menu_allroots, _("Roots of &Polynomial"),
                              _("Find all roots of a polynomial"),
                              wxITEM_NORMAL);
+  wxglade_tmp_menu_3->Append(menu_bfallroots, _("Roots of Polynomial (bfloat)"),
+                               _("Find all roots of a polynomial (bfloat)"),
+                               wxITEM_NORMAL);
   wxglade_tmp_menu_3->Append(menu_realroots, _("&Roots of Polynomial (Real)"),
                              _("Find real roots of a polynomial"),
                              wxITEM_NORMAL);
@@ -406,6 +403,9 @@ void wxMaximaFrame::SetupMenu()
   wxglade_tmp_menu_4->Append(menu_gen_mat, _("&Generate Matrix..."),
                              _("Generate a matrix from a 2-dimensional array"),
                              wxITEM_NORMAL);
+  wxglade_tmp_menu_4->Append(menu_gen_mat_lambda, _("Generate Matrix from Expression..."),
+                               _("Generate a matrix from a lambda expression"),
+                               wxITEM_NORMAL);
   wxglade_tmp_menu_4->Append(menu_enter_mat, _("&Enter Matrix..."),
                              _("Enter a matrix"), wxITEM_NORMAL);
   wxglade_tmp_menu_4->Append(menu_invert_mat, _("&Invert Matrix"),
@@ -423,7 +423,7 @@ void wxMaximaFrame::SetupMenu()
                              _("Find eigenvectors of a matrix"),
                              wxITEM_NORMAL);
   wxglade_tmp_menu_4->Append(menu_adjoint_mat, _("Ad&joint Matrix"),
-                             _("Compute the adjoint maxrix"), wxITEM_NORMAL);
+                             _("Compute the adjoint matrix"), wxITEM_NORMAL);
   wxglade_tmp_menu_4->Append(menu_transpose, _("&Transpose Matrix"),
                              _("Transpose a matrix"), wxITEM_NORMAL);
   wxglade_tmp_menu_4->AppendSeparator();
@@ -452,6 +452,9 @@ void wxMaximaFrame::SetupMenu()
   wxglade_tmp_menu_6->Append(menu_limit, _("Find &Limit..."),
                              _("Find a limit of an expression"),
                              wxITEM_NORMAL);
+  wxglade_tmp_menu_6->Append(menu_lbfgs, _("Find Minimum..."),
+                               _("Find a (unconstrained) minimum of an expression"),
+                               wxITEM_NORMAL);
   wxglade_tmp_menu_6->Append(menu_series, _("Get &Series..."),
                              _("Get the Taylor or power series of expression"),
                              wxITEM_NORMAL);
@@ -594,7 +597,7 @@ void wxMaximaFrame::SetupMenu()
   wxglade_tmp_menu_6 = new wxMenu;
   wxglade_tmp_menu_6->Append(menu_num_out, _("Toggle &Numeric Output"),
                              _("Toggle numeric output"), wxITEM_NORMAL);
-  wxglade_tmp_menu_6->Append(menu_to_float, _("To &Float\tCtrl-F"),
+  wxglade_tmp_menu_6->Append(menu_to_float, _("To &Float"),
                              _("Calculate float value of the last result"),
                              wxITEM_NORMAL);
   wxglade_tmp_menu_6->Append(menu_to_bfloat, _("To &Bigfloat"),
@@ -618,6 +621,9 @@ void wxMaximaFrame::SetupMenu()
   APPEND_MENU_ITEM(wxglade_tmp_menu_7, menu_show_tip, _("Show &Tips..."),
                    _("Show a tip"), wxART_TIP);
   wxglade_tmp_menu_7->AppendSeparator();
+  wxglade_tmp_menu_7->Append(menu_help_tutorials, _("Tutorials"),
+                             _("Online tutorials"), wxITEM_NORMAL);
+  wxglade_tmp_menu_7->AppendSeparator();
   wxglade_tmp_menu_7->Append(menu_build_info, _("Build &Info"),
                              _("Info about Maxima build"), wxITEM_NORMAL);
   wxglade_tmp_menu_7->Append(menu_bug_report, _("&Bug Report"),
@@ -631,7 +637,7 @@ void wxMaximaFrame::SetupMenu()
 #else
           _("About wxMaxima"),
 #endif
-                   _("About wxMaxima"), wxT("stock_about"));
+          _("About wxMaxima"), wxT("stock_about"));
   frame_1_menubar->Append(wxglade_tmp_menu_7, _("&Help"));
 
   SetMenuBar(frame_1_menubar);
@@ -670,19 +676,19 @@ void wxMaximaFrame::SetupToolBar()
                            IMAGE("configure.png"),
                            _("Configure wxMaxima"));
   frame_1_toolbar->AddSeparator();
+  frame_1_toolbar->AddTool(tb_cut, _("Cut"),
+                           IMAGE("cut.png"),
+                           _("Cut selection"));
   frame_1_toolbar->AddTool(tb_copy, _("Copy"),
                            IMAGE("copy.png"),
                            _("Copy selection"));
-  frame_1_toolbar->AddTool(tb_delete, _("Delete"),
-                           IMAGE("cut.png"),
-                           _("Delete selection"));
+  frame_1_toolbar->AddTool(tb_paste, _("Paste"),
+                           IMAGE("paste.png"),
+                           _("Paste from clipboard"));
   frame_1_toolbar->AddSeparator();
-  frame_1_toolbar->AddTool(tb_insert_text, _("Insert text"),
-                           IMAGE("text.png"),
-                           _("Insert text"));
-  frame_1_toolbar->AddTool(tb_insert_input, _("Insert input group"),
-                           IMAGE("input.png"),
-                           _("Insert input cell"));
+  frame_1_toolbar->AddTool(tb_find, _("Find"),
+                             IMAGE("find.png"),
+                             _("Find and replace"));
   frame_1_toolbar->AddSeparator();
   frame_1_toolbar->AddTool(tb_interrupt, _("Interrupt"),
                            IMAGE("stop.png"),
@@ -732,23 +738,23 @@ void wxMaximaFrame::SetupToolBar()
                                                     wxART_TOOLBAR),
                            _("Configure wxMaxima"));
   frame_1_toolbar->AddSeparator();
+  frame_1_toolbar->AddTool(tb_cut, _("Cut"),
+                           wxArtProvider::GetBitmap(wxT("gtk-cut"),
+                                                    wxART_TOOLBAR),
+                           _("Cut selection"));
   frame_1_toolbar->AddTool(tb_copy, _("Copy"),
                            wxArtProvider::GetBitmap(wxT("gtk-copy"),
                                                     wxART_TOOLBAR),
                            _("Copy selection"));
-  frame_1_toolbar->AddTool(tb_delete, _("Delete"),
-                           wxArtProvider::GetBitmap(wxT("gtk-cut"),
+  frame_1_toolbar->AddTool(tb_paste, _("Paste"),
+                           wxArtProvider::GetBitmap(wxT("gtk-paste"),
                                                     wxART_TOOLBAR),
-                           _("Delete selection"));
+                           _("Paste from clipboard"));
   frame_1_toolbar->AddSeparator();
-  frame_1_toolbar->AddTool(tb_insert_text, _("Insert text"),
-                           wxArtProvider::GetBitmap(wxART_NORMAL_FILE,
+  frame_1_toolbar->AddTool(tb_find, _("Find..."),
+                           wxArtProvider::GetBitmap(wxT("gtk-find"),
                                                     wxART_TOOLBAR),
-                           _("Insert text"));
-  frame_1_toolbar->AddTool(tb_insert_input, _("Insert input cell"),
-                           wxArtProvider::GetBitmap(wxART_EXECUTABLE_FILE,
-                                                    wxART_TOOLBAR),
-                           _("Insert input cell"));
+                           _("Find and replace"));
   frame_1_toolbar->AddSeparator();
   frame_1_toolbar->AddTool(tb_interrupt, _("Interrupt"),
                            wxArtProvider::GetBitmap(wxT("gtk-stop"),
@@ -771,6 +777,7 @@ void wxMaximaFrame::SetupToolBar()
                                                     wxART_TOOLBAR),
                            _("Show Maxima help"));
   frame_1_toolbar->Realize();
+
   SetToolBar(frame_1_toolbar);
 }
 
@@ -795,13 +802,13 @@ void wxMaximaFrame::SaveRecentDocuments()
   wxConfigBase *config = wxConfig::Get();
 
   // Delete previous recent documents
-  for (int i=0; i<10; i++) {
+  for (unsigned int i=0; i<10; i++) {
     wxString recent = wxString::Format(wxT("RecentDocuments/document_%d"), i);
     config->DeleteEntry(recent);
   }
 
   // Save new recent documents
-  for (int i=0; i<m_recentDocuments.Count(); i++) {
+  for (unsigned int i=0; i<m_recentDocuments.Count(); i++) {
     wxString recent = wxString::Format(wxT("RecentDocuments/document_%d"), i);
     config->Write(recent, m_recentDocuments[i]);
   }
@@ -809,15 +816,13 @@ void wxMaximaFrame::SaveRecentDocuments()
 
 void wxMaximaFrame::UpdateRecentDocuments()
 {
-  wxMenu* fileMenu = GetMenuBar()->GetMenu(GetMenuBar()->FindMenu(_("&File")));
-
   for (int i=menu_recent_document_0; i<= menu_recent_document_9; i++)
   {
     wxMenuItem *item = m_recentDocumentsMenu->Remove(i);
     delete item;
     item = NULL;
 
-    if (i-menu_recent_document_0 < m_recentDocuments.Count())
+    if (i-menu_recent_document_0 < (signed)m_recentDocuments.Count())
     {
       wxFileName filename(m_recentDocuments[i - menu_recent_document_0]);
       wxString path(filename.GetPath()), fullname(filename.GetFullName());
@@ -844,3 +849,189 @@ void wxMaximaFrame::RemoveRecentDocument(wxString file)
 
   UpdateRecentDocuments();
 }
+
+bool wxMaximaFrame::IsPaneDisplayed(int id)
+{
+  bool displayed = false;
+
+  switch (id) {
+    case menu_pane_math:
+      displayed = m_manager.GetPane(wxT("math")).IsShown();
+      break;
+    case menu_pane_history:
+      displayed = m_manager.GetPane(wxT("history")).IsShown();
+      break;
+    case menu_pane_stats:
+      displayed = m_manager.GetPane(wxT("stats")).IsShown();
+      break;
+    case menu_pane_format:
+      displayed = m_manager.GetPane(wxT("format")).IsShown();
+      break;
+  }
+
+  return displayed;
+}
+
+void wxMaximaFrame::ShowPane(int id, bool show)
+{
+  switch (id) {
+    case menu_pane_math:
+      m_manager.GetPane(wxT("math")).Show(show);
+      break;
+    case menu_pane_history:
+      m_manager.GetPane(wxT("history")).Show(show);
+      break;
+    case menu_pane_stats:
+      m_manager.GetPane(wxT("stats")).Show(show);
+      break;
+    case menu_pane_format:
+      m_manager.GetPane(wxT("format")).Show(show);
+      break;
+    case menu_pane_hideall:
+      m_manager.GetPane(wxT("math")).Show(false);
+      m_manager.GetPane(wxT("history")).Show(false);
+      m_manager.GetPane(wxT("stats")).Show(false);
+      m_manager.GetPane(wxT("format")).Show(false);
+      break;
+  }
+
+  m_manager.Update();
+}
+
+wxPanel* wxMaximaFrame::CreateMathPane()
+{
+  wxGridSizer *grid = new wxGridSizer(2);
+  wxPanel *panel = new wxPanel(this, -1);
+
+  int style = wxALL | wxEXPAND;
+#if defined __WXMSW__
+  int border = 1;
+#else
+  int border = 0;
+#endif
+
+  grid->Add(new wxButton(panel, button_ratsimp, _("Simplify")), 0, style, border);
+  grid->Add(new wxButton(panel, button_radcan, _("Simplify (r)")), 0, style, border);
+  grid->Add(new wxButton(panel, button_factor, _("Factor")), 0, style, border);
+  grid->Add(new wxButton(panel, button_expand, _("Expand")), 0, style, border);
+  grid->Add(new wxButton(panel, button_rectform, _("Rectform")), 0, style, border);
+  grid->Add(new wxButton(panel, button_subst, _("Subst...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_trigrat, _("Canonical (tr)")), 0, style, border);
+  grid->Add(new wxButton(panel, button_trigsimp, _("Simplify (tr)")), 0, style, border);
+  grid->Add(new wxButton(panel, button_trigexpand, _("Expand (tr)")), 0, style, border);
+  grid->Add(new wxButton(panel, button_trigreduce, _("Reduce (tr)")), 0, style, border);
+  grid->Add(new wxButton(panel, button_solve, _("Solve...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_solve_ode, _("Solve ODE...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_diff, _("Diff...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_integrate, _("Integrate...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_limit, _("Limit...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_taylor, _("Series...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_plot2, _("Plot 2D...")), 0, style, border);
+  grid->Add(new wxButton(panel, button_plot3, _("Plot 3D...")), 0, style, border);
+
+  panel->SetSizer(grid);
+  grid->Fit(panel);
+  grid->SetSizeHints(panel);
+
+  return panel;
+}
+
+wxPanel* wxMaximaFrame::CreateStatPane()
+{
+  wxGridSizer *grid1 = new wxGridSizer(2);
+  wxBoxSizer *box = new wxBoxSizer(wxVERTICAL);
+  wxBoxSizer *box1 = new wxBoxSizer(wxVERTICAL);
+  wxGridSizer *grid2 = new wxGridSizer(2);
+  wxGridSizer *grid3 = new wxGridSizer(2);
+  wxBoxSizer *box3 = new wxBoxSizer(wxVERTICAL);
+  wxPanel *panel = new wxPanel(this, -1);
+
+  int style = wxALL | wxEXPAND;
+#if defined __WXMSW__
+  int border = 1;
+#else
+  int border = 0;
+#endif
+  int sizerBorder = 2;
+
+  grid1->Add(new wxButton(panel, menu_stats_mean, _("Mean")), 0, style, border);
+  grid1->Add(new wxButton(panel, menu_stats_median, _("Median")), 0, style, border);
+  grid1->Add(new wxButton(panel, menu_stats_var, _("Variance")), 0, style, border);
+  grid1->Add(new wxButton(panel, menu_stats_dev, _("Deviation")), 0, style, border);
+
+  box->Add(grid1, 0, style, sizerBorder);
+
+  box1->Add(new wxButton(panel, menu_stats_tt1, _("Mean Test...")), 0, style, border);
+  box1->Add(new wxButton(panel, menu_stats_tt2, _("Mean Difference Test...")), 0, style, border);
+  box1->Add(new wxButton(panel, menu_stats_tnorm, _("Normality Test")), 0, style, border);
+  box1->Add(new wxButton(panel, menu_stats_linreg, _("Linear Regression")), 0, style, border);
+  box1->Add(new wxButton(panel, menu_stats_lsquares, _("Least Squares Fit...")), 0, style, border);
+
+  box->Add(box1, 0, style, sizerBorder);
+
+  grid2->Add(new wxButton(panel, menu_stats_histogram, _("Histogram...")), 0, style, border);
+  grid2->Add(new wxButton(panel, menu_stats_scatterplot, _("Scatterplot...")), 0, style, border);
+  grid2->Add(new wxButton(panel, menu_stats_barsplot, _("Barsplot")), 0, style, border);
+  grid2->Add(new wxButton(panel, menu_stats_piechart, _("Piechart")), 0, style, border);
+  grid2->Add(new wxButton(panel, menu_stats_boxplot, _("Boxplot")), 0, style, border);
+
+  box->Add(grid2, 0, style, sizerBorder);
+
+  grid3->Add(new wxButton(panel, menu_stats_readm, _("Read Matrix...")), 0, style, border);
+  grid3->Add(new wxButton(panel, menu_stats_enterm, _("Enter Matrix...")), 0, style, border);
+
+  box->Add(grid3, 0, style, sizerBorder);
+
+  box3->Add(new wxButton(panel, menu_stats_subsample, _("Subsample...")), 0, style, border);
+
+  box->Add(box3, 0, style, sizerBorder);
+
+  panel->SetSizer(box);
+  box->Fit(panel);
+  box->SetSizeHints(panel);
+
+  return panel;
+}
+
+wxPanel *wxMaximaFrame::CreateFormatPane()
+{
+  wxGridSizer *grid = new wxGridSizer(2);
+  wxPanel *panel = new wxPanel(this, -1);
+
+  int style = wxALL | wxEXPAND;
+#if defined __WXMSW__
+  int border = 1;
+#else
+  int border = 0;
+#endif
+
+  grid->Add(new wxButton(panel, menu_format_text, _("Text")), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_title, _("Title")), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_subsection, _("Subsection")), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_section, _("Section")), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_image, _("Image")), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_pagebreak, _("Pagebreak")), 0, style, border);
+
+  panel->SetSizer(grid);
+  grid->Fit(panel);
+  grid->SetSizeHints(panel);
+
+  return panel;
+}
+
+void wxMaximaFrame::ShowToolBar(bool show)
+{
+  if (show) {
+    if (GetToolBar() == NULL)
+      SetupToolBar();
+  }
+  else {
+    wxToolBar *tbar = GetToolBar();
+    if (tbar != NULL) {
+      delete tbar;
+      m_plotSlider = NULL;
+      SetToolBar(NULL);
+    }
+  }
+}
+
