@@ -1,5 +1,5 @@
 ///
-///  Copyright (C) 2004-2007 Andrej Vodopivec <andrejv@users.sourceforge.net>
+///  Copyright (C) 2004-2009 Andrej Vodopivec <andrejv@users.sourceforge.net>
 ///
 ///  This program is free software; you can redistribute it and/or modify
 ///  it under the terms of the GNU General Public License as published by
@@ -26,9 +26,11 @@
 #include <wx/listbox.h>
 #include <wx/bmpbuttn.h>
 #include <wx/arrstr.h>
+#include <wx/aui/aui.h>
 
 #include "MathCtrl.h"
 #include "Setup.h"
+#include "History.h"
 
 enum {
   socket_client_id = wxID_HIGHEST,
@@ -44,8 +46,10 @@ enum {
   menu_interrupt_id,
   menu_restart_id,
   menu_solve,
+  menu_solve_to_poly,
   menu_solve_num,
   menu_allroots,
+  menu_bfallroots,
   menu_realroots,
   menu_solve_lin,
   menu_solve_algsys,
@@ -55,6 +59,7 @@ enum {
   menu_ivp_2,
   menu_bvp,
   menu_gen_mat,
+  menu_gen_mat_lambda,
   menu_enter_mat,
   menu_invert_mat,
   menu_cpoly,
@@ -100,6 +105,7 @@ enum {
   menu_partfrac,
   menu_sum,
   menu_limit,
+  menu_lbfgs,
   menu_series,
   menu_pade,
   menu_map,
@@ -128,6 +134,7 @@ enum {
   button_trigsimp,
   button_trigexpand,
   button_trigreduce,
+  button_trigrat,
   button_integrate,
   button_diff,
   button_sum,
@@ -151,8 +158,7 @@ enum {
   menu_show_tip,
   menu_copy_from_console,
   menu_copy_tex_from_console,
-  menu_copy_input_from_console,
-  menu_cut_input_from_console,
+  menu_copy_text_from_console,
   menu_undo,
   menu_delete_selection,
   menu_select_all,
@@ -162,8 +168,14 @@ enum {
   menu_to_gamma,
   menu_texform,
   button_enter,
-  menu_inc_fontsize,
-  menu_dec_fontsize,
+  menu_zoom_in,
+  menu_zoom_out,
+  menu_zoom_80,
+  menu_zoom_100,
+  menu_zoom_120,
+  menu_zoom_150,
+  menu_zoom_200,
+  menu_zoom_300,
   menu_copy_as_bitmap,
   menu_copy_to_file,
   menu_export_html,
@@ -173,21 +185,26 @@ enum {
   tb_open,
   tb_save,
   tb_copy,
-  tb_delete,
+  tb_paste,
+  tb_cut,
   tb_print,
   tb_pref,
   tb_interrupt,
   tb_help,
-  tb_insert_text,
-  tb_insert_input,
   tb_animation_start,
   tb_animation_stop,
+  tb_find,
 #endif
   menu_evaluate,
   menu_add_comment,
+  menu_add_subsection,
   menu_add_section,
   menu_add_title,
+  menu_add_pagebreak,
   menu_insert_input,
+  menu_insert_previous_input,
+  menu_autocomplete,
+  menu_autocomplete_templates,
   menu_cut,
   menu_paste,
   menu_paste_input,
@@ -208,9 +225,45 @@ enum {
   menu_recent_document_7,
   menu_recent_document_8,
   menu_recent_document_9,
-  menu_insert_image
+  menu_insert_image,
+  menu_pane_hideall,
+  menu_pane_math,
+  menu_pane_history,
+  menu_pane_inspector,
+  menu_pane_format,
+  menu_pane_stats,
+  menu_stats_mean,
+  menu_stats_median,
+  menu_stats_var,
+  menu_stats_dev,
+  menu_stats_tt1,
+  menu_stats_tt2,
+  menu_stats_tnorm,
+  menu_stats_linreg,
+  menu_stats_lsquares,
+  menu_stats_histogram,
+  menu_stats_scatterplot,
+  menu_stats_barsplot,
+  menu_stats_piechart,
+  menu_stats_boxplot,
+  menu_stats_readm,
+  menu_stats_enterm,
+  menu_stats_subsample,
+  menu_format_code,
+  menu_format_text,
+  menu_format_subsection,
+  menu_format_section,
+  menu_format_title,
+  menu_format_image,
+  menu_format_pagebreak,
+  menu_help_tutorials,
+  menu_show_toolbar,
+  menu_edit_find
 };
 
+#define FIRST_PANE menu_pane_hideall
+#define LAST_PANE  menu_pane_stats
+
 class wxMaximaFrame: public wxFrame
 {
 public:
@@ -218,10 +271,15 @@ public:
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 long style = wxDEFAULT_FRAME_STYLE);
+  ~wxMaximaFrame();
   void UpdateRecentDocuments();
   void AddRecentDocument(wxString file);
   void RemoveRecentDocument(wxString file);
   wxString GetRecentDocument(int i) { return m_recentDocuments[i]; }
+  bool IsPaneDisplayed(int id);
+  void ShowPane(int id, bool hide);
+  void AddToHistory(wxString cmd) { m_history->AddToHistory(cmd); }
+  void ShowToolBar(bool show);
 private:
   void set_properties();
   void do_layout();
@@ -229,32 +287,17 @@ private:
   void SetupToolBar();
 #endif
   void SetupMenu();
+  wxPanel *CreateStatPane();
+  wxPanel *CreateMathPane();
+  wxPanel *CreateFormatPane();
 protected:
   void LoadRecentDocuments();
   void SaveRecentDocuments();
+  wxAuiManager m_manager;
   wxMenuBar* frame_1_menubar;
-  wxPanel *panel;
+
   MathCtrl* m_console;
-  wxButton* button_2;
-  wxButton* button_3;
-  wxButton* button_4;
-  wxButton* button_5;
-  wxButton* button_6;
-  wxButton* button_7;
-  wxButton* button_8;
-  wxButton* button_9;
-  wxButton* button_10;
-  wxButton* button_11;
-  wxButton* button_12;
-  wxButton* button_13;
-  wxButton* button_14;
-  wxButton* button_15;
-  wxButton* button_16;
-  wxButton* button_17;
-  wxButton* button_18;
-  wxButton* button_19;
-  wxButton* button_20;
-  wxButton* button_21;
+  History * m_history;
   wxStatusBar* frame_1_statusbar;
   wxToolBar* frame_1_toolbar;
   wxSlider* m_plotSlider;
